import {
  browser, by, element, ElementArrayFinder, ElementFinder, protractor,
  ProtractorExpectedConditions
} from 'protractor';

/**
 * Trida s public funkcemi pro ziskavani elementu z obrazovky
 */
export class AppPage {

  private EC: ProtractorExpectedConditions = protractor.ExpectedConditions;
  private timeout: number = 5000;
  /**
   * prechod na pozadovanou stranku
   * @returns {wdpromise.Promise<any>}
   */
  public navigateTo(url: string): wdpromise.Promise<any> {
    return browser.get(url);
  }

  /**
   * ziskani elementu podle id
   * @param id elementu
   * @returns {ElementFinder}
   */
  public getElementById(id: string): ElementFinder {
    let el = element(by.id(id));
    browser.wait(this.EC.presenceOf(el), this.timeout);
    return el;
  }

  public getElementByCss(css: string): ElementFinder {
    let el = element(by.css(css));
    browser.wait(this.EC.presenceOf(el), this.timeout);
    return el;
  }

  /**
   * ziskani tlacitka z DOMu pomoci id
   * @param id tlacitka
   * @returns {ElementFinder}
   */
  public getButtonById(id: string): ElementFinder {
    let btn = element(by.id(id));
    browser.wait(this.EC.elementToBeClickable(btn), this.timeout);
    return btn;
  }

  /**
   * ziskani seznamu elementu podle css
   * @param css
   * @returns {ElementArrayFinder}
   */
  public getListByCss(css: string): ElementArrayFinder {
    let elArr = element.all(by.css(css));
    browser.wait(this.EC.presenceOf(elArr), this.timeout);
    return elArr;
  }

  /**
   * ziskani inputu podle jmena
   * @param name inputu
   * @returns {ElementFinder}
   */
  public getInputByName(name: string): ElementFinder {
    let input = element(by.name(name));
    // pri castych pokusech chce google vyplnit captcha kod, proto je tu cas na rucni vyplneni
    browser.wait(this.EC.visibilityOf(input), 6 * this.timeout);
    return input;
  }

  /**
   * ceka na to, kdy element bude obsahovat urcity text (napr kdyz cekam na nacteni dat)
   * @param id elementu
   * @param text ktery element musi obsahovat
   * @returns {ElementFinder}
   */
  public waitForElementWithText(id: string, text: string): ElementFinder {
    let el = element(by.id(id));
    browser.wait(this.EC.textToBePresentInElement(el, text), this.timeout);
    return el;

  }
}
