import { Component, ElementRef, ViewChild } from "@angular/core";
import { Observable } from "rxjs/Rx";
import { Alert } from "./alert";

@Component({
  selector: "app-alert",
  templateUrl: "./alert.component.html",
  styleUrls: ["./alert.component.css"]
})
export class AlertComponent {

  private static instance: AlertComponent;

  public static success(message: string): Alert {
    return AlertComponent.instance.createAlert(message, Alert.success);
  }

  public static info(message: string): Alert {
    return AlertComponent.instance.createAlert(message, Alert.info);
  }

  public static error(message: string): Alert {
    return AlertComponent.instance.createAlert(message, Alert.error);
  }

  public static warning(message: string): Alert {
    return AlertComponent.instance.createAlert(message, Alert.warning);
  }

  public static loading(message: string, type: string = Alert.info): Alert {
    return AlertComponent.instance.createAlert(message, type, true);
  }

  @ViewChild("alertsList")
  public alertsList: ElementRef;
  public alerts: Array<Alert> = [];

  constructor() {
    AlertComponent.instance = this;
  }

  /**
   * vyhodi alert okno, ktere po 2 sec zmizi
   * @param message
   * @param type (error, info, warning, loading, success)
   * @param isLoading
   * @returns {Alert}
   */
  private createAlert(message: string, type: string, isLoading: boolean = false): Alert {
    const alert: Alert = new Alert(message, type, isLoading, this);
    AlertComponent.instance.alerts.unshift(alert);
    this.alertsList.nativeElement.classList.add("add-new");
    Observable.timer(2000).subscribe(() => {
      this.alertsList.nativeElement.classList.remove("add-new");
    });
    return alert;
  }
}
