import * as $ from 'jquery';
import { BrowserModule } from '@angular/platform-browser';
import { NgModule } from '@angular/core';
import { AppComponent } from './app.component';
import {AngularFireStorageModule} from "angularfire2/storage";
import {AngularFireAuthModule} from "angularfire2/auth";
import {AngularFireDatabaseModule} from "angularfire2/database";
import {AngularFirestoreModule} from "angularfire2/firestore";
import {AngularFireModule} from "angularfire2";
import {environment} from "../environments/environment";
import {FormsModule} from "@angular/forms";
import {BrowserAnimationsModule} from "@angular/platform-browser/animations";
import {MatIconModule, MatMenuModule} from "@angular/material";
import { ButtonsModule, ModalModule} from "ngx-bootstrap";
import { AlertComponent } from './alert/alert.component';
import { ConfirmComponent } from './confirm/confirm.component';
import { FilesComponent } from './files/files.component';
import { LogsComponent } from './logs/logs.component';
import { LoginComponent } from './login/login.component';
import { MainComponent } from './main/main.component';
import { PhotoComponent } from './photo/photo.component';
import { TestsComponent } from './tests/tests.component';
import { TextComponent } from './text/text.component';
import { TimelineComponent } from './timeline/timeline.component';
import { TimelineVideoComponent } from './timeline-video/timeline-video.component';
import { VideoComponent } from './video/video.component';
import {AngularfireService} from "../services/angularfire.service";
import {GoldenLayoutService} from "../services/golden-layout.service";
import {TestService} from "../services/test.service";
import {TimelineService} from "../services/timeline.service";
import {UserService} from "../services/user.service";
import {VideoService} from "../services/video.service";
import {routes} from "./app.router";
import {RouterModule} from "@angular/router";
import {ConfirmController} from "./confirm/confirm.controller";
import { InfoComponent } from './info/info.component';
import { QuestComponent } from './quest/quest.component';
import { TestCasesComponent } from './test-cases/test-cases.component';
import { ParticipantsComponent } from './participants/participants.component';
import { MembersComponent } from './members/members.component';
import { MarkersComponent } from './markers/markers.component';
import { QuestInstComponent } from './quest-inst/quest-inst.component';
import { TestCasesInstComponent } from './test-cases-inst/test-cases-inst.component';

export const verze: string = "0.0.16";

@NgModule({
  declarations: [
    AppComponent,
    AlertComponent,
    ConfirmComponent,
    FilesComponent,
    LogsComponent,
    LoginComponent,
    MainComponent,
    PhotoComponent,
    TestsComponent,
    TextComponent,
    TimelineComponent,
    TimelineVideoComponent,
    VideoComponent,
    InfoComponent,
    QuestComponent,
    TestCasesComponent,
    ParticipantsComponent,
    MembersComponent,
    MarkersComponent,
    QuestInstComponent,
    TestCasesInstComponent
  ],
  imports: [
    BrowserModule,
    AngularFireModule.initializeApp(environment.firebase),
    AngularFirestoreModule,
    AngularFirestoreModule.enablePersistence(),
    AngularFireDatabaseModule,
    AngularFireAuthModule,
    AngularFireStorageModule,
    FormsModule,
    BrowserAnimationsModule,
    MatIconModule,
    MatMenuModule,
    ButtonsModule.forRoot(),
    ModalModule.forRoot(),
    RouterModule.forRoot(routes),
  ],
  entryComponents: [
    MainComponent,
    TimelineComponent,
    TestsComponent,
    LogsComponent,
    TextComponent,
    PhotoComponent,
    ConfirmComponent,
    VideoComponent,
    InfoComponent,
    QuestComponent,
    QuestInstComponent,
    TestCasesComponent,
    TestCasesInstComponent,
    ParticipantsComponent,
    MembersComponent,
    MarkersComponent
  ],
  providers: [
    AngularfireService,
    GoldenLayoutService,
    TestService,
    TimelineService,
    UserService,
    VideoService,
    ConfirmController
  ],
  bootstrap: [AppComponent]
})
export class AppModule { }
