import {Component, OnDestroy, OnInit, TemplateRef} from "@angular/core";
import {UserService} from "../../services/user.service";
import {Router} from "@angular/router";
import {User} from "../../model/User";
import {TestItem} from "../../model/TestItem";
import {TestInst} from "../../model/TestInst";
import {GoldenLayoutService} from "../../services/golden-layout.service";
import {ConfirmController} from "../confirm/confirm.controller";
import {Locale} from "../../constants/Locale";
import {BsModalRef, BsModalService} from "ngx-bootstrap";
import {AngularfireService} from "../../services/angularfire.service";
import {TestService} from "../../services/test.service";

@Component({
  selector: "app-files",
  templateUrl: "./files.component.html",
  styleUrls: ["./files.component.css"]
})
export class FilesComponent implements OnInit, OnDestroy {


  public test: any;
  public testData: any = {};
  public testInst: any = [];
  public selectedItem: any;
  public selectedTest: any;
  public selectedInst: any;
  public menuType = "simple";
  public modalRef: BsModalRef;
  public newName: string;
  public isOwner = false;
  public locale: any = Locale.getLocale();
  public testFolderRoll = true;
  public testInstsRoll = false;
  public testFilesFolderRoll = false;
  private user: User;
  private subscriptions: Array<any> = [];

  constructor(private userSvc: UserService,
              private router: Router,
              private glSvc: GoldenLayoutService,
              private confirm: ConfirmController,
              private modalService: BsModalService,
              private testSvc: TestService,
              private afService: AngularfireService) {
    this.user = this.userSvc.user;
    if (this.user === undefined) {
      this.router.navigate(["login"]);
      return;
    }
  }

  ngOnInit(): void {
    try {
      console.log(this.test);

      // Ziskani members z databaze, kvuli pravum (member vidi mene veci nez owner)
      const subscMember = this.afService.getTestMembers(this.test.id).subscribe((members: Array<any>) => {
        console.log(members);
        if (!members) {
          return;
        }

        this.testSvc.setMembers(members);

        const member = members.find((value) => value.id === this.user.uid);
        if (member) {

          if (member.role === "owner") {
            this.isOwner = true;
            // zikani dat testu
            const subscTests = this.afService.getTest(this.test.id).subscribe(data => {
              console.log(data);
              if (!data) {
                return;
              }
              this.testData = {};
              const info: any = {};
              const participants: any = [];
              const instances: any = [];
              const files: any = [];
              const markers: any = [];
              const members: any = [];
              const testCases: any = [];
              let preTestQuest: any = {};
              let postTestQuest: any = {};

              data["info"]["id"] = this.test.id;

              if (data["info"]) {
                info["archived"] = data["info"]["archived"];
                info["notes"] = data["info"]["info"];
                info["name"] = data["info"]["name"];
                info["timestamp"] = data["info"]["timestamp"];
                info["id"] = data["info"]["id"];
                preTestQuest = data["info"]["pre_test_questionnaire"];
                postTestQuest = data["info"]["post_test_questionnaire"];

                // participants
                for (const key in data["info"]["participants"]) {
                  if (!data["info"]["participants"].hasOwnProperty(key)) {
                    continue;
                  }
                  const participant = data["info"]["participants"][key];
                  participants.push({
                    id: participant.id,
                    name: participant["name"],
                  });
                }

                // test cases
                for (const key in data["info"]["test_cases"]) {
                  if (!data["info"]["test_cases"].hasOwnProperty(key)) {
                    continue;
                  }
                  const tc = data["info"]["test_cases"][key];
                  testCases.push({
                    id: tc.id,
                    name: tc["name"],
                    type: tc["type"],
                  });
                }
              }

              // files
              for (const key in data["data"]) {
                if (!data["data"].hasOwnProperty(key)) {
                  continue;
                }
                const value = data["data"][key];
                const file = {
                  id: value.id,
                  name: value["name"],
                  fileName: "",
                  type: value["type"],
                };
                if (value["type"] === "PHOTO") {
                  this.afService.getAbstractData(value.id).subscribe(photo => {
                    if (photo && photo["photo_link"] && photo["photo_link"]["fileName"]) {
                      file.fileName = photo["photo_link"]["fileName"];
                    }
                  });
                }
                if (value["type"] === "VIDEO") {
                  this.afService.getAbstractData(value.id).subscribe(photo => {
                    if (photo && photo["video"] && photo["video"]["fileName"]) {
                      file.fileName = photo["video"]["fileName"];
                    }
                  });
                }
                files.push(file);
              }


              // instances
              for (const key in data["instances"]) {
                if (!data["instances"].hasOwnProperty(key)) {
                  continue;
                }
                const value = data["instances"][key];
                instances.push({
                  id: value.id,
                  name: value["name"],
                  test_definition_id: value["test_definition_id"],
                  timestamp: value["timestamp"],
                });
              }

              // markers
              for (const key in data["markers"]) {
                if (!data["markers"].hasOwnProperty(key)) {
                  continue;
                }
                const value = data["markers"][key];
                markers.push({
                  id: value.id,
                  order: value["order"],
                  text: value["text"],
                });
              }

              // members
              for (const key in data["members"]) {
                if (!data["members"].hasOwnProperty(key)) {
                  continue;
                }
                const value = data["members"][key];
                members.push({
                  id: value.id,
                  email: value["email"],
                  role: value["role"],
                });
              }

              this.testData = {
                info: info,
                participants: participants,
                testCases: testCases,
                instances: instances,
                markers: markers,
                members: members,
                files: files,
                quest: {
                  preTestQuest: preTestQuest,
                  postTestQuest: postTestQuest,
                }
              };
            });
            const subscInst = this.afService.getAllInstances(this.test.id).subscribe(data => {
              if (!data) {
                return;
              }

              this.testInst = [];

              // prochazeni instanci
              for (const key in data) {
                if (!data.hasOwnProperty(key)) {
                  continue;
                }
                // urcita instance
                const value = data[key];
                const info: any = {};
                const participants: any = [];
                let testCase: any = {};
                const files: any = [];
                let preTestQuest: any = {};
                let postTestQuest: any = {};

                info["id"] = value["id"];
                info["name"] = value["name"];
                info["notes"] = value["notes"];
                info["participant"] = value["participant"];
                info["test_definition_id"] = value["test_definition_id"];

                preTestQuest = value["pre_test_questionnaire"];
                postTestQuest = value["post_test_questionnaire"];
                testCase = value["test_case"];
                // data instance
                for (const k in value["data"]) {
                  if (!value["data"].hasOwnProperty(k)) {
                    continue;
                  }
                  const v = value["data"][k];
                  files.push({
                    id: v.id,
                    name: v["name"],
                    type: v["type"],
                  });
                }

                const item = {
                  info: info,
                  files: files,
                  testCase: testCase,
                  quest: {
                    preTestQuest: preTestQuest,
                    postTestQuest: postTestQuest,
                  }
                };
                if (this.selectedInst && item.info.id === this.selectedInst.info.id) {
                  item["folderRoll"] = this.selectedInst.folderRoll;
                  item["filesFolderRoll"] = this.selectedInst.filesFolderRoll;
                  this.selectedInst = undefined;
                }
                this.testInst.push(item);
              }

            });
            this.subscriptions.push(subscTests);
            this.subscriptions.push(subscInst);
          } else {
            // uzivatel je jen member
            this.isOwner = false;
            const sub = this.afService.getTestInfoAll(this.test.id).subscribe((infoData: any) => {
              if (!infoData) {
                return;
              }
              console.log(infoData);
              this.testData = {};
              const info: any = {};
              const testCases: any = [];
              const participants: any = [];
              let preTestQuest: any = {};
              let postTestQuest: any = {};


              info["archived"] = infoData["archived"];
              info["notes"] = infoData["info"];
              info["name"] = infoData["name"];
              info["timestamp"] = infoData["timestamp"];
              info["id"] = this.test.id;
              preTestQuest = infoData["pre_test_questionnaire"];
              postTestQuest = infoData["post_test_questionnaire"];

              // participants
              for (const key in infoData["participants"]) {
                if (!infoData["participants"].hasOwnProperty(key)) {
                  continue;
                }
                const participant = infoData["participants"][key];
                participants.push({
                  id: participant.id,
                  name: participant["name"],
                });
              }

              // test cases
              for (const key in infoData["test_cases"]) {
                if (!infoData["test_cases"].hasOwnProperty(key)) {
                  continue;
                }
                const tc = infoData["test_cases"][key];
                testCases.push({
                  id: tc.id,
                  name: tc["name"],
                  type: tc["type"],
                });
              }
              this.testData = {
                info: info,
                participants: participants,
                testCases: testCases,
                instances: [],
                markers: [],
                members: [],
                files: [],
                quest: {
                  preTestQuest: preTestQuest,
                  postTestQuest: postTestQuest,
                }
              };
              console.log(this.testData);
            });
            this.subscriptions.push(sub);
          }
        }
      });


      this.subscriptions.push(subscMember);
    } catch (e) {
      console.log("NGINIT ERROR:" + e);
    }
  }

  public openTestInfo(testId: string, inst: any): void {
    this.glSvc.showInfo(testId, inst);
  }

  public openTestQuest(quest: any, testId: string): void {
    this.glSvc.showQuest(quest, testId);
  }

  public openTestCases(cases: any, testId: string): void {
    this.glSvc.showCases(cases, testId);
  }

  public openTestCasesInst(inst: any): void {
    this.glSvc.showCasesInst(inst);
  }

  public openParticipants(testId: any): void {
    this.glSvc.showParticipants(testId);
  }

  public openMembers(testId: any): void {
    this.glSvc.showMembers(testId);
  }

  public openMarkers(testId: any): void {
    this.glSvc.showMarkers(testId);
  }

  public openInstQuest(inst: any): void {
    this.glSvc.showQuestInst(inst.quest, this.test.id, inst);
  }

  /**
   * otevre video a napoji ho na timeline (bude se ovladat pomoci timeline, muze se jich pridat vice)
   */
  public openVideoTimeline(): void {
    console.log("open video");
    this.glSvc.focusItem(this.selectedItem, this.selectedInst);
    this.glSvc.showVideoTimeline(this.selectedItem, this.selectedInst);
  }

  /**
   * v zavislosti na typu souboru, otevre okno a nacte data
   * @param item info o souboru, co se ma otevrit
   * @param inst je zadana, pokud se soubor otevre v instanci a ne v testu
   */
  openFile(item: TestItem, inst: any = null): void {
    switch (item["type"]) {
      case "LOG":
        this.glSvc.showLog(item, inst);
        break;
      case "TEXT":
        this.glSvc.showText(item, inst);
        break;
      case "VIDEO":
        if (inst) {
          this.selectedItem = item;
          this.selectedInst = inst;
          this.openVideoTimeline();
        } else {
          this.glSvc.showVideoSingle(item);
        }
        break;
      case "PHOTO":
        this.glSvc.showPhoto(item, inst);
        break;
    }
  }

  /**
   * video se otevre samostatne, bez napojeni na timeline
   * @param item video k zobrazeni
   * @param inst pokud je zadana, video se otevre samo v instanci, jinak v testu (v All Files)
   */
  public openVideoSingle(item: TestItem, inst: any): void {
    this.glSvc.showVideo(item, inst);
  }

  public addLogFile(): void {
    if (this.menuType === "filesFolderInst") {
      // vyvoreni logu v instanci
      this.afService.createFileInst(this.test, this.selectedInst.info.id, "New Untitled Log", "LOG");
    } else {
      // vyvoreni logu v testu
      this.afService.createFile(this.test, "New Untitled Log", "LOG");
    }
  }

  public addTextFile(): void {
    if (this.menuType === "filesFolderInst") {
      // vyvoreni textu v instanci
      this.afService.createFileInst(this.test, this.selectedInst.info.id, "New Untitled Document", "TEXT");
    } else {
      // vyvoreni txtu v testu
      this.afService.createFile(this.test, "New Untitled Document", "TEXT");
    }
  }

  public addPhotoFile(fileList: FileList): void {
    console.log("adding foto");

    if (this.menuType === "filesFolderInst") {
      if (fileList.length > 0) {
        // vyvoreni fotky v instanci
        this.afService.uploadImg(fileList[0], this.test, this.selectedInst.info.id);
      }
    } else {
      if (fileList.length > 0) {
        // vyvoreni fotky v testu
        this.afService.uploadImg(fileList[0], this.test, undefined);
      }
    }
  }

  public addVideoFile(fileList: FileList): void {
    if (this.menuType === "filesFolderInst") {
      if (fileList.length > 0) {
        // vyvoreni videa v instanci
        this.afService.uploadVideo(fileList[0], this.test, this.selectedInst.info.id);
      }
    } else {
      if (fileList.length > 0) {
        // vyvoreni videa v testu
        this.afService.uploadVideo(fileList[0], this.test, undefined);
      }
    }
  }

  /**
   * smaze hodnotu v inputu a otevre ho (pro pridani fotky, videa)
   * @param input videa nebo fotky
   */
  public openInput(input: any): void {
    input.value = "";
    input.click();
  }

  /**
   * pri kliknuti na soubor se nastavi promenne
   * @param test ktery je otevreny (vzdy jeden stejny)
   * @param instance ve ktere je soubor
   * @param type typ souboru
   * @param item soubor
   */
  public setSelectedItem(test: any, instance: any, type: string, item?: any): void {
    this.selectedTest = test;
    this.selectedInst = instance;
    this.selectedItem = item;
    this.menuType = type;
  }

  /**
   * zobrazeni popup okna pro prejmenovani souboru
   * @param template HTML sablona
   */
  public rename(template: TemplateRef<any>): void {
    // podle toho co chceme prejmenovat, toho jmeno se vezme (a pak prejmenuje)
    if (this.menuType === "rootFolder") {
      this.newName = this.selectedTest.name;
    } else if (this.menuType === "instance") {
      this.newName = this.selectedInst.info.name;
    } else {
      this.newName = this.selectedItem.name;
    }
    this.openModale(template);
  }

  /**
   * otevreni popup okna
   * @param template HTML sablona
   */
  public openModale(template: TemplateRef<any>): void {
    this.modalRef = this.modalService.show(template);
  }

  public closeModale(): void {
    this.selectedItem = undefined;
    this.newName = "";
    this.modalRef.hide();
  }

  /**
   * ulozeni noveho jmena do databaze
   */
  public saveRename(): void {
    console.log("rename");
    if (this.menuType === "rootFolder") {
      this.selectedTest.name = this.newName;
      this.afService.renameTest(this.selectedTest);
    }
    if (this.menuType === "instance") {
      this.selectedInst.info.name = this.newName;
      this.afService.renameInst(this.selectedTest, this.selectedInst);
    }
    if (this.menuType === "fileTest") {
      this.selectedItem.name = this.newName;
      this.afService.renameFileTest(this.selectedTest, this.selectedItem);
    }
    if (this.menuType === "fileInst") {
      this.selectedItem.name = this.newName;
      this.afService.renameFileInst(this.selectedTest, this.selectedInst, this.selectedItem);
    }
    this.closeModale();
  }

  /**
   * samazani souboru
   * @param test ze ktereho se maze
   * @param inst ve ktere je soubor
   * @param item soubor
   * @returns {Promise<void>}
   */
  public async remove(test, inst, item): Promise<void> {
    if (item && await this.confirm.show(this.locale.deleteSure + item.name, this.locale.delete)) {
      // soubor je v instanci
      if (this.menuType === "fileInst") {
        console.log("mazu instanci: " + item);
        this.afService.removeItemInst(test.id, inst.info.id, item.id, item.type).then(() => {
          this.glSvc.closeById(item.id);
        });
      } else {
        // soubor je v testu
        this.afService.removeItem(test.id, item).then(() => {
          this.glSvc.closeById(item.id);
        });
      }
    }
  }

  public addInstance(): void {
    console.log(this.newName);
    this.afService.createInst(this.test, this.newName);
    this.closeModale();
  }

  public openInst(): void {
    this.glSvc.openInst(this.selectedInst);
  }

  /**
   * odrolovani/zarolovani adresare instance
   */
  public testInstRoll(instance: TestInst): void {
    instance.folderRoll = !instance.folderRoll;
    this.selectedInst = instance;
    this.openInst();
  }

  ngOnDestroy(): void {
    this.subscriptions.forEach(sub => {
      sub.unsubscribe();
    });
  }
}
