import {Component, OnDestroy, OnInit} from "@angular/core";
import {Subject} from "rxjs/Subject";
import {Locale} from "../../constants/Locale";
import {AngularfireService} from "../../services/angularfire.service";
import {AlertComponent} from "../alert/alert.component";

@Component({
  selector: "app-info",
  templateUrl: "./info.component.html",
  styleUrls: ["./info.component.css"]
})
export class InfoComponent implements OnInit, OnDestroy {

  public data: any;
  public text: any;
  public dataShow: string;
  public originalText: string;
  public subscriptions: Array<any> = [];
  public timestamp: number;
  public editing = false;
  public locale: any = Locale.getLocale();
  private isInst = false;

  constructor(private afService: AngularfireService) { }

  ngOnInit(): void {
    console.log(this.data);
    if (!this.data) {return; }

    let sub;

    // jedna se o info instance
    if (this.data.instId) {
      this.isInst = true;
      sub = this.afService.getInstInfo(this.data.testId, this.data.instId).subscribe((info: any) => {
        console.log(info);
        this.dataShow = info;
      });
      this.subscriptions.push(sub);

    } else {
      // jedna se o info o testu
      this.isInst = false;

      sub = this.afService.getTestInfo(this.data.testId).subscribe((info: any) => {
        this.dataShow = info;
        console.log(info);
      });
      this.subscriptions.push(sub);
    }
  }

  public edit(): void {
    this.editing = true;
    this.originalText = this.dataShow;
  }

  public async save(): Promise<void> {
    console.log("save");
    if (this.isInst) {
      // info instance
      this.afService.updateInstInfo(this.data.testId, this.data.instId, this.dataShow);
    } else {
      // info testu
      this.afService.updateTestInfo(this.data.testId, this.dataShow);
    }
    this.editing = false;
  }

  public cancel(): void {
    console.log("cancel");
    this.dataShow = this.originalText;
    this.editing = false;
  }

  ngOnDestroy(): void {
    try {
      this.subscriptions.forEach(sub => {
        sub.unsubscribe();
      });
    } catch (e) {
      console.log(e);
    }
  }
}
