import {Component, OnDestroy, OnInit} from "@angular/core";
import {AngularfireService} from "../../services/angularfire.service";
import {Marker} from "../../model/Marker";
import {VideoService} from "../../services/video.service";
import {ConfirmController} from "../confirm/confirm.controller";
import {Locale} from "../../constants/Locale";
import {AlertComponent} from "../alert/alert.component";
import {Subject} from "rxjs/Subject";

@Component({
  selector: "app-log",
  templateUrl: "./logs.component.html",
  styleUrls: ["./logs.component.css"]
})
export class LogsComponent implements OnInit, OnDestroy {

  public log: any;
  public selectedMarker = "";
  public newLogComment = "";
  public arrLog: any = [];
  public locale: any = Locale.getLocale();
  public markers: Array<Marker> = [];
  public currentTime = 0;
  public test: any;
  private subscriptions: Array<any> = [];
  private originalTime: any;
  private originalMarker: any;
  private originalComment: any;
  private timelineTimeSubject: Subject<any> = new Subject<any>();

  constructor(private afService: AngularfireService, private confirm: ConfirmController, private videoSvc: VideoService) {
    this.videoSvc.getTimelineSub().subscribe((data: any) => {
      this.currentTime = data.time;
    });
    this.timelineTimeSubject = this.videoSvc.getFindItemInTimeline();
  }

  ngOnInit() {
    console.log("init log-cont: " + this.log);
    if (!this.log) {
      return;
    }
    // ziskani markeru testu
    const markersSub = this.afService.getMarkers(this.test.id).subscribe((marks: any) => {
      if (!marks) {
        return;
      }
      this.markers = marks;
      console.log(marks);
    });
    this.subscriptions.push(markersSub);

    // ziskani dat pro dany log soubor
    const url = this.afService.getAbstractData(this.log["id"]);
    const urlSub = url.subscribe((data): any => {
      if (!data) {
        return;
      }
      this.arrLog = [];
      const logs = data["data"];


      for (const key in logs) {
        if (!logs.hasOwnProperty(key)) {
          continue;
        }
        const marker = logs[key];
        this.arrLog.push({
          id: marker.id,
          marker: marker["marker"],
          timestamp: Number.parseInt(marker["timestamp"]),
          comment: marker["comment"],
        });
      }
      this.arrLog.sort(function (a, b) {
        a = parseInt(a["timestamp"]);
        b = parseInt(b["timestamp"]);
        if (a === b) {
          return 1;
        }
        return b - a;
      });
    });
    this.subscriptions.push(urlSub);
  }

  /**
   * zalozeni logu
   */
  public addLog(): void {
    if (!this.log || !this.log.id) {
      return;
    }

    // TODO dat do service
    this.afService.setData("/abstract_data/" + this.log.id + "/data",
      {
        comment: this.newLogComment,
        marker: this.selectedMarker,
        timestamp: this.videoSvc.getCurrentTime() + "",
        id: ""
      })
      .then(resp => {
        this.newLogComment = "";
      });
    this.selectedMarker = "";
  }

  /**
   * smazani logu z log souboru
   * @param log k smazani
   * @returns {Promise<void>}
   */
  public async deleteLog(log: any): Promise<void> {
    if (await this.confirm.show(this.locale.deleteSure + log.marker, this.locale.delete)) {
      this.afService.deleteData("/abstract_data/" + this.log.id + "/data/" + log.id).then(() => {
        AlertComponent.info(this.locale.deleteOk);
      }).catch(error => {
        console.log(error);
        AlertComponent.error(this.locale.deleteFail);
      });
    }
  }

  /**
   * cas v timeline se nastavi na cas logu (prejede kurzor timeline)
   * @param log
   */
  public findInTimeline(log: any): void {
    this.timelineTimeSubject.next(log["timestamp"]);
  }

  public editLog(log: any): void {
    log.editingPre = true;
    this.originalComment = log.comment;
    this.originalMarker = log.marker;
    this.originalTime = log.timestamp;
  }

  /**
   * zruseni editace, zmeny se vrati
   * @param log
   */
  public cancelLog(log: any): void {
    log.comment = this.originalComment;
    log.marker = this.originalMarker;
    log.timestamp = this.originalTime;
    log.editingPre = false;
  }

  public saveLog(log: any): void {
    console.log(log);
    this.afService.editLog("/abstract_data/" + this.log.id + "/data/" + log.id, log)
      .then(resp => {
        log.editingPre = false;
      });
  }

  ngOnDestroy(): void {
    this.subscriptions.forEach(sub => {
      try {
        sub.unsubscribe();
      } catch (e) {
      }
    });
  }
}
