import {Component, ElementRef, HostListener, OnDestroy, OnInit, ViewChild, ViewContainerRef} from "@angular/core";
import {ActivatedRoute, Router} from "@angular/router";
import {UserService} from "../../services/user.service";
import {GoldenLayoutService} from "../../services/golden-layout.service";

@Component({
  selector: "app-main-container",
  templateUrl: "./main.component.html",
  styleUrls: ["./main.component.css"]
})
export class MainComponent implements OnInit, OnDestroy {

  @ViewChild("layout") private container: ElementRef;
  private test: any;
  private subscriptions: Array<any> = [];

  constructor(private glSvc: GoldenLayoutService,
              private userSvc: UserService,
              private router: Router,
              private viewContainer: ViewContainerRef,
              private route: ActivatedRoute) {
    if (this.userSvc.user === undefined) {
      this.router.navigate(["login"]);
      return;
    }
    // data poslana z predesle obrazovky
    this.test = JSON.parse(this.route.snapshot.paramMap.get("test"));

  }

  ngOnInit() {
    this.glSvc.initLayout(this.test, this.container, this.viewContainer);
  }

  ngOnDestroy(): void {
    this.subscriptions.forEach(sub => {
      try {
        sub.unsubscribe();
      } catch (e) {
      }
    });
  }

  /**
   * pokud se zmeni velikost okna prohlizece, golden layout se prizpusobi
   * @param event
   */
  @HostListener("window:resize", ["$event"])
  onResize(event) {
    console.log("resize");
    if (this.glSvc.layout) {
      this.glSvc.layout.updateSize();
    }
  }
}
