import {Component, OnDestroy, OnInit} from "@angular/core";
import {AngularfireService} from "../../services/angularfire.service";

@Component({
  selector: "app-markers",
  templateUrl: "./markers.component.html",
  styleUrls: ["./markers.component.css"]
})
export class MarkersComponent implements OnInit, OnDestroy {

  public markers: Array<any> = [];
  public data: any;
  private subscriptions: Array<any> = [];
  private testId: string;

  constructor(private afService: AngularfireService) {
  }

  ngOnInit() {
    console.log(this.data);
    if (!this.data) {
      return;
    }

    const sub = this.afService.getTestMarkers(this.data.testId).subscribe((markers: any) => {
      markers.sort((a: any, b: any) => {
        if (a.order < b.order) {
          return -1;
        } else if (a.order > b.order) {
          return 1;
        } else {
          return 0;
        }
      });
      this.markers = markers;
    });
    this.subscriptions.push(sub);
  }

  /**
   * vytvoreni markeru
   * @param input z ktereho se vezme nazev
   */
  public add(input): void {
    const order = (this.markers && this.markers.length > 0) ? this.markers[this.markers.length - 1].order + 1 : 0;

    this.afService.addMarker(this.testId, order, input.value).then(() => {
      input.value = "";
    });
  }

  public edit(marker: any): void {
    marker.editing = true;
    marker.textNew = marker.text;
  }

  public save(marker: any): void {
    this.afService.editMarker(this.testId, marker.id, marker.textNew);
  }

  public close(marker: any): void {
    marker.editing = false;
  }

  public delete(marker: any): void {
    this.afService.delMarker(this.testId, marker.id);
  }

  ngOnDestroy(): void {
    try {
      this.subscriptions.forEach(sub => {
        sub.unsubscribe();
      });
    } catch (e) {
      console.log(e);
    }
  }
}
