import {Component, Input, OnDestroy, OnInit} from "@angular/core";
import {AngularfireService} from "../../services/angularfire.service";
import {UserService} from "../../services/user.service";

@Component({
  selector: "app-members",
  templateUrl: "./members.component.html",
  styleUrls: ["./members.component.css"]
})
export class MembersComponent implements OnInit, OnDestroy {

  public members: Array<any> = [];
  public users: Array<any> = [];
  public newUser: any;
  public user: any;
  public testId: string;
  public newUserExist = false;
  public type = "owner";
  public newUserStr: string ;
  public data: any;
  private subscriptions: Array<any> = [];

  constructor(private afService: AngularfireService, private userSvc: UserService) { }

  ngOnInit() {
    console.log(this.data);
    if (!this.data) {return; }

    this.user = this.userSvc.getUser();
    console.log(this.user);

    // seznam uzivatelu pro dany test
    const sub2 = this.afService.getUsers().subscribe((users: any) => {
      this.users = [];
      for (const id in users) {
        this.users.push({
          id: id,
          email: users[id]["email"],
          name: users[id]["name"],
        });
      }
    });
    this.subscriptions.push(sub2);

    const sub = this.afService.getTestMembers(this.data.testId).subscribe((members: any) => {
      this.members = members;
    });
    this.subscriptions.push(sub);
  }

  /**
   * kontrola, jestli pozadovany email ma pristup do databaze (nejdriv s musi prihlasit)
   * kontroluje se po kazdem stisku klavesy
   * @param input
   */
  checkUser(input: any) {
    this.newUser = this.users.find(u => u.email === this.newUserStr);
    if (this.newUser) {
      this.newUserExist = true;
      input.classList.remove("input-fail");
      input.classList.add("input-ok");
    } else {
      this.newUserExist = false;
      input.classList.add("input-fail");
      input.classList.remove("input-ok");
    }
  }

  /**
   * prida uzivatele mezi members, pokud existuje v databazi
   */
  public add(): void {
    if (!this.newUserExist) {
      return;
    }
    this.afService.addOwner(this.data.testId, this.newUser, this.type);
    this.newUserStr = "";
  }

  public edit(member: any): void {
    member.editing = true;
    member.newRole = member.role;
    console.log(member);
  }

  /**
   * ulozi upravy (zmena prav)
   * @param member
   */
  public save(member: any): void {
    this.afService.addOwner(this.data.testId, member, member.newRole);
  }

  public close(member: any): void {
    member.editing = false;
  }

  /**
   * vymaze uzivatele z testu (nebude mit zadna prava)
   * @param member
   */
  public delete(member: any): void {
    console.log(member);
    this.afService.delMember(this.testId, member);
  }

  ngOnDestroy(): void {
    try {
      this.subscriptions.forEach(sub => {
        sub.unsubscribe();
      });
    } catch (e) {
      console.log(e);
    }
  }
}
