import {Component, ElementRef, OnDestroy, OnInit} from "@angular/core";
import {AngularfireService} from "../../services/angularfire.service";
import {Locale} from "../../constants/Locale";
import {Subject} from "rxjs/Subject";
import {VideoService} from "../../services/video.service";

@Component({
  selector: "app-photo",
  templateUrl: "./photo.component.html",
  styleUrls: ["./photo.component.css"]
})
export class PhotoComponent implements OnInit, OnDestroy {

  public style: object = {};
  public url: string;
  public locale: any = Locale.getLocale();
  public data: any;
  private timestamp: number;
  private subscriptions: Array<any> = [];
  private timelineTimeSubject: Subject<any> = new Subject<any>();

  constructor(private afService: AngularfireService, private videoSvc: VideoService) {
    this.timelineTimeSubject = this.videoSvc.getFindItemInTimeline();
  }

  ngOnInit() {
    console.log(this.data);
    const url = this.afService.getAbstractData(this.data["id"]);
    const urlSub = url.subscribe((data): any => {
      console.log(data);
      if (data && data["photo_link"]) {
        this.url = data["photo_link"]["downloadUrl"];
        this.timestamp = Number.parseInt(data["timestamp"]);
      }
    });
    this.subscriptions.push(urlSub);
  }

  /**
   * nastavi cas timeline na cas fotky
   */
  public findInTimeline(): void {
    this.timelineTimeSubject.next(this.timestamp);
  }

  /**
   * zvetseni/zmenseni fotky
   * @param pixels
   * @param element
   */
  public zoom(pixels: number, element: ElementRef): void {
    let w: any = element["style"]["width"];
    w = w.substring(0, w.length - 2);
    w = Number.parseInt(w);
    w += pixels;
    element["style"]["width"] = w + "px";
  }

  ngOnDestroy(): void {
    try {
      this.subscriptions.forEach(sub => {
        sub.unsubscribe();
      });
    } catch (e) {
      console.log(e);
    }
  }
}
