import {Component, OnDestroy, OnInit} from "@angular/core";
import {AngularfireService} from "../../services/angularfire.service";

@Component({
  selector: "app-quest",
  templateUrl: "./quest-inst.component.html",
  styleUrls: ["./quest-inst.component.css"]
})
export class QuestInstComponent implements OnInit, OnDestroy {

  public data: any;
  public newNamePre = "";
  public newNamePost = "";
  public pre: any = {id: "", name: "", answer: "", data: []};
  public post: any = {id: "", name: "", answer: "", data: []};
  public editingPre = false;
  public editingPost = false;
  private subscriptions: Array<any> = [];

  constructor(private afService: AngularfireService) {
  }

  ngOnInit() {
    console.log(this.data);
    if (!this.data) {
      return;
    }
    const sub = this.afService.getInstInfoQuest(this.data.testId, this.data.instId).subscribe((info: any) => {
      console.log(info);
      const pre = info.pre_test_questionnaire;
      const post = info.post_test_questionnaire;

      if (pre) {
        console.log(pre);
        const preSub = this.afService.getAbstractDataList(pre.id).subscribe((preData): any => {
          console.log(preData);
          preData.sort((a: any, b: any) => {
            if (a.order < b.order) {
              return -1;
            } else if (a.order > b.order) {
              return 1;
            } else {
              return 0;
            }
          });
          this.pre = {id: pre.id, name: pre.name, answer: pre.answer, data: preData};
        });
        this.subscriptions.push(preSub);
      }
      if (post) {
        console.log(post);
        const postSub = this.afService.getAbstractDataList(post.id).subscribe((postData): any => {
          postData.sort((a: any, b: any) => {
            if (a.order < b.order) {
              return -1;
            } else if (a.order > b.order) {
              return 1;
            } else {
              return 0;
            }
          });
          this.post = {id: post.id, name: post.name, answer: post.answer, data: postData};
          console.log(postData);
        });
        this.subscriptions.push(postSub);
      }

    });
    this.subscriptions.push(sub);
  }

  /**
   * smaze dotaznik i s odpovedemi
   * @param parent
   * @param isPre pre/post dotaznik
   */
  public deleteAll(parent: any, isPre: boolean): void {
    this.afService.delQuestInst(this.data.testId, this.data.instId, parent.id, isPre ? "pre_test_questionnaire" : "post_test_questionnaire")
      .then(() => {
        if (isPre) {
          this.pre = {id: "", name: "", answer: "", data: []};
        } else {
          this.post = {id: "", name: "", answer: "", data: []};
        }
      });

  }

  /**
   * editace odpovedi na otazku
   * @param quest
   * @param parent
   */
  public editQuest(quest: any, parent: any): void {
    quest.editing = true;
    quest.ansNew = quest.answer;
    console.log(quest);
  }

  /**
   * ulozeni odpovedi na otazku
   * @param quest
   * @param parent
   */
  public saveQuest(quest: any, parent: any): void {
    quest.editing = false;
    this.afService.editQuestAns(parent.id, quest.id, quest.ansNew);
  }

  /**
   * zrusni editace
   * @param quest
   */
  public closeQuest(quest: any): void {
    quest.editing = false;
  }

  /**
   * editace dotazniku
   * @param isPre
   */
  public edit(isPre: boolean): void {
    if (isPre) {
      this.editingPre = true;
      this.newNamePre = this.pre.name;
    } else {
      this.editingPost = true;
      this.newNamePost = this.post.name;
    }
  }

  /**
   * zavrani editace dotazniku
   * @param isPre
   */
  public close(isPre: boolean): void {
    if (isPre) {
      this.editingPre = false;
    } else {
      this.editingPost = false;
    }
  }

  /**
   * prejmenovani dotazniku
   * @param newName
   * @param quest
   * @param isPre
   */
  public saveName(newName: any, quest: any, isPre: boolean): void {
    this.afService.updateQuestionnareNameInst(this.data.testId, this.data.instId, quest.id, newName, isPre ? "pre_test_questionnaire" : "post_test_questionnaire")
      .then(() => {
        if (isPre) {
          this.editingPre = false;
        } else {
          this.editingPost = false;
        }
      });

  }

  /**
   * vytvoreni dotazniku pro odpovedi
   * @param questType
   */
  public create(questType: string): void {
    if (questType === "pre") {
      this.afService.createQuestInst(this.data.testId, this.data.instId, "pre_test_questionnaire");
    } else {
      this.afService.createQuestInst(this.data.testId, this.data.instId, "post_test_questionnaire");
    }
  }

  ngOnDestroy(): void {
    try {
      this.subscriptions.forEach(sub => {
        sub.unsubscribe();
      });
    } catch (e) {
      console.log(e);
    }
  }
}
