import {Component, OnDestroy, OnInit} from "@angular/core";
import {AngularfireService} from "../../services/angularfire.service";

@Component({
  selector: "app-quest",
  templateUrl: "./quest.component.html",
  styleUrls: ["./quest.component.css"]
})
export class QuestComponent implements OnInit, OnDestroy {

  public data: any;
  public newNamePre = "";
  public newNamePost = "";
  public pre: any = {id: "", name: "", data: []};
  public post: any = {id: "", name: "", data: []};
  public editingPre = false;
  public editingPost = false;
  private subscriptions: Array<any> = [];

  constructor(private afService: AngularfireService) {
  }

  ngOnInit() {
    console.log(this.data);
    if (!this.data) {
      return;
    }
    const sub = this.afService.getTestInfoAll(this.data.testId).subscribe((info: any) => {
      console.log(info);
      const pre = info.pre_test_questionnaire;
      const post = info.post_test_questionnaire;

      // pre-test dotaznik
      if (pre) {
        const preSub = this.afService.getAbstractDataList(pre.id).subscribe((preData): any => {
          console.log(preData);
          preData.sort((a: any, b: any) => {
            if (a.order < b.order) {
              return -1;
            } else if (a.order > b.order) {
              return 1;
            } else {
              return 0;
            }
          });
          this.pre = {id: pre.id, name: pre.name, data: preData};
        });
        this.subscriptions.push(preSub);
      } else {
        this.pre = {id: "", name: "", data: []};
      }

      // post-test dotaznik
      if (post) {
        const postSub = this.afService.getAbstractDataList(post.id).subscribe((postData): any => {
          postData.sort((a: any, b: any) => {
            if (a.order < b.order) {
              return -1;
            } else if (a.order > b.order) {
              return 1;
            } else {
              return 0;
            }
          });
          this.post = {id: post.id, name: post.name, data: postData};
          console.log(postData);
        });
        this.subscriptions.push(postSub);
      } else {
        this.post = {id: "", name: "", data: []};
      }
    });
    this.subscriptions.push(sub);
  }

  /**
   * smaze cely dotaznik i s otazkami
   * @param quest dotaznik
   * @param questType pre/post
   */
  public deleteAll(quest: any, questType: string): void {
    if (questType === "pre") {
      this.afService.deleteQuestAll(quest.id, this.data.testId, "pre_test_questionnaire");
    } else {
      this.afService.deleteQuestAll(quest.id, this.data.testId, "post_test_questionnaire");
    }
  }

  /**
   * vytvoreni otazky v dotazniku
   * @param input - z inputu se vezme jmeno
   * @param parent - pre nebo post test dotaznik
   */
  public addQuest(input: any, parent: any): void {
    const order = (parent.data && parent.data.length > 0) ? parent.data[parent.data.length - 1].order + 1 : 0;
    this.afService.addQuest(parent.id, order, input.value)
      .then(() => {
        input.value = "";
      });
  }

  /**
   * smaze otazku
   * @param quest
   * @param parent
   */
  public delQuest(quest: any, parent: any): void {
    this.afService.delQuest(parent.id, quest.id);

  }

  /**
   * editace otazky
   * @param quest
   * @param parent
   */
  public editQuest(quest: any, parent: any): void {
    quest.editing = true;
    quest.textNew = quest.text;
    console.log(quest);
  }

  /**
   * ulozeni zmen v otazce
   * @param quest
   * @param parent
   */
  public saveQuest(quest: any, parent: any): void {
    quest.editing = false;
    this.afService.editQuest(parent.id, quest.id, quest.textNew);
  }

  /**
   * zavreni editace
   * @param quest
   */
  public closeQuest(quest: any): void {
    quest.editing = false;
  }

  /**
   * vytvori dotaznik
   * @param questType
   */
  public create(questType: string): void {
    if (questType === "pre") {
      this.afService.createQuest(this.data.testId, "pre_test_questionnaire");
    } else {
      this.afService.createQuest(this.data.testId, "post_test_questionnaire");
    }
  }

  /**
   * editace dotazniku (ne otazky)
   * @param isPre pre/post
   */
  public edit(isPre: boolean): void {
    if (isPre) {
      this.editingPre = true;
      this.newNamePre = this.pre.name;
    } else {
      this.editingPost = true;
      this.newNamePost = this.post.name;
    }
  }

  /**
   * zaavre editace dotazniku
   * @param isPre
   */
  public close(isPre: boolean): void {
    if (isPre) {
      this.editingPre = false;
    } else {
      this.editingPost = false;
    }
  }

  /**
   * ulozeni nazvu dotazniku
   * @param newName nove jmeno
   * @param quest dotaznik
   * @param isPre pre/post
   */
  public saveName(newName: any, quest: any, isPre: boolean): void {
    this.afService.updateQuestionnareNameTest(this.data.testId, quest.id, newName, isPre ? "pre_test_questionnaire" : "post_test_questionnaire")
      .then(() => {
        if (isPre) {
          this.editingPre = false;
        } else {
          this.editingPost = false;
        }
      });

  }

  ngOnDestroy(): void {
    try {
      this.subscriptions.forEach(sub => {
        sub.unsubscribe();
      });
    } catch (e) {
      console.log(e);
    }
  }
}
