import {Component, OnDestroy, OnInit} from "@angular/core";
import {AngularfireService} from "../../services/angularfire.service";

@Component({
  selector: "app-test-cases",
  templateUrl: "./test-cases.component.html",
  styleUrls: ["./test-cases.component.css"]
})
export class TestCasesComponent implements OnInit, OnDestroy {

  public data: any;
  public cases: Array<any> = [];
  public editing = false;
  public selectedCaseArr: Array<any>;
  public selectedCaseId = "";
  public selectedSub: any;
  private subscriptions: Array<any> = [];

  constructor(private afService: AngularfireService) {
  }

  ngOnInit() {
    console.log(this.data);
    if (!this.data) {
      return;
    }

    // ziskam info tesstu, uvnitr jsou test cases (uvnitr test case jsou tasky)
    const sub = this.afService.getTestInfoAll(this.data.testId).subscribe((info: any) => {
      console.log(info);
      const cases = info.test_cases;
      this.cases = [];

      for (const id in cases) {
        const item = {
          id: id,
          name: cases[id].name,
          tasks: []
        };
        this.cases.push(item);
      }
      console.log(this.cases);
    });
    this.subscriptions.push(sub);
  }

  /* -------- CASES -------- */

  /**
   * prida test case
   * @param input s nazvem
   */
  public addCase(input): void {
    console.log(input.value, this.cases);
    this.afService.addCase(this.data.testId, input.value)
      .then(() => {
        input.value = "";
        if (this.selectedSub) {
          this.selectedSub.unsubscribe();
        }
        this.selectedCaseArr = [];
        this.selectedCaseId = "";
      });
  }

  /**
   * vyber test case (kliknuti na nej),
   * pri vyberu test case se nactou jeho tasky
   * @param casse
   */
  public selectCase(casse: any): void {
    console.log("select", casse.id);
    console.log(this.cases);
    event.stopPropagation();
    // pokud kliknu na input, tak se tato funkce zrusi
    if ((event.target as any).nodeName === "INPUT") {
      return;
    }

    if (this.selectedCaseId === casse.id) {
      this.selectedCaseArr = [];
      this.selectedCaseId = "";
    } else {
      if (this.selectedSub) {
        this.selectedSub.unsubscribe();
      }
      this.selectedSub = this.afService.getAbstractDataList(casse.id).subscribe((tasks: any) => {
        console.log("jsem v " + tasks);
        tasks.sort((a: any, b: any) => {
          if (a.order < b.order) {
            return -1;
          } else if (a.order > b.order) {
            return 1;
          } else {
            return 0;
          }
        });
        this.selectedCaseArr = tasks;
      });
      this.selectedCaseId = casse.id;
    }
  }

  public editCase(item: any): void {
    // pokud kliknu na toto tlacitko, nechci aby se event propagoval na dalis elementy
    event.stopPropagation();
    item.editing = true;
    item.nameNew = item.name;
    console.log(item);
  }

  public saveCase(item: any): void {
    // pokud kliknu na toto tlacitko, nechci aby se event propagoval na dalis elementy
    event.stopPropagation();
    item.editing = false;
    this.afService.editCase(this.data.testId, item.id, item.nameNew);
  }

  public closeCase(item: any): void {
    // pokud kliknu na toto tlacitko, nechci aby se event propagoval na dalis elementy
    event.stopPropagation();
    item.editing = false;
  }

  public delCase(item: any): void {
    // pokud kliknu na toto tlacitko, nechci aby se event propagoval na dalis elementy
    event.stopPropagation();
    this.afService.delCase(this.data.testId, item.id).then(() => {
      if (this.selectedSub) {
        this.selectedSub.unsubscribe();
      }
      this.selectedCaseArr = [];
      this.selectedCaseId = "";
    });
  }

  /* -------- TASKS -------- */

  public addTask(input): void {
    console.log(input.value);
    const order = (this.selectedCaseArr && this.selectedCaseArr.length > 0)
      ? this.selectedCaseArr[this.selectedCaseArr.length - 1].order + 1 : 0;

    this.afService.addTask(this.selectedCaseId, order, input.value).then(() => {
      input.value = "";
    });
  }

  public delTask(item: any): void {
    console.log("delete task");
    this.afService.delTask(this.selectedCaseId, item.id);
  }

  public editTask(item: any): void {
    item.editing = true;
    item.nameNew = item.text;
    console.log(item);
  }

  public saveTask(item: any): void {
    item.editing = false;
    console.log(item);
    this.afService.editTask(this.selectedCaseId, item.id, item.nameNew);
  }

  public closeTask(item: any): void {
    item.editing = false;
  }

  ngOnDestroy(): void {
    try {
      this.subscriptions.forEach(sub => {
        sub.unsubscribe();
      });
    } catch (e) {
      console.log(e);
    }
  }
}
