import {Component, OnDestroy, OnInit} from "@angular/core";
import {TestItem} from "../../model/TestItem";
import {AngularfireService} from "../../services/angularfire.service";
import {Router} from "@angular/router";
import {UserService} from "../../services/user.service";

@Component({
  selector: "app-test-selection",
  templateUrl: "./tests.component.html",
  styleUrls: ["./tests.component.css"]
})
export class TestsComponent implements OnInit, OnDestroy {

  public testsFavourite: Array<TestItem> = [];
  public testsActive: Array<TestItem> = [];
  public testsArchived: Array<TestItem> = [];
  private subscriptions: Array<any> = [];

  constructor(private userSvc: UserService, private router: Router, private afService: AngularfireService) {
    if (this.userSvc.user === undefined) {
      this.router.navigate(["login"]);
      return;
    }

    // stazeni vsech testu ke kterym ma uzivatel pristup a rozdelim je podle typu
    const userTestsData = this.afService.getUserTests(this.userSvc.user.uid);
    const subsc = userTestsData.subscribe(keys => {
      if (!keys) {
        return;
      }
      this.testsFavourite = [];
      this.testsActive = [];
      this.testsArchived = [];
      let keyArr = [];
      if (keys["favourite"]) {
        keyArr = Object.keys(keys["favourite"]).map(function (x) {
          return keys["favourite"][x];
        });
        for (const k of keyArr) {
          this.testsFavourite.push(this.objectToTest(k));
        }
      }
      if (keys["active"]) {
        keyArr = Object.keys(keys["active"]).map(function (x) {
          return keys["active"][x];
        });
        for (const k of keyArr) {
          this.testsActive.push(this.objectToTest(k));
        }
      }
      if (keys["archived"]) {
        keyArr = Object.keys(keys["archived"]).map(function (x) {
          return keys["archived"][x];
        });
        for (const k of keyArr) {
          this.testsArchived.push(this.objectToTest(k));
        }
      }
    });
    this.subscriptions.push(subsc);
  }

  /**
   * prevedeni Object na TestItem
   * @param object
   * @returns {TestItem}
   */
  private objectToTest(object: any): TestItem {
    const test: TestItem = {
      id: object["id"],
      name: object["name"],
      role: object["role"],
      timestamp: object["timestamp"] < 0 ? -object["timestamp"] : object["timestamp"]
    };
    return test;
  }

  /**
   * otevreni tstu, prejdu na hlavni obrazovku s testem
   * @param test
   */
  public openTest(test: TestItem) {
    this.router.navigate(["main", {test: JSON.stringify(test)}]);
  }

  ngOnInit() {
    console.log("test page");
  }

  ngOnDestroy(): void {
    this.subscriptions.forEach(sub => {
      sub.unsubscribe();
    });
  }

}
