import {Component, OnDestroy, OnInit} from "@angular/core";
import {AngularfireService} from "../../services/angularfire.service";
import {Locale} from "../../constants/Locale";
import {Subject} from "rxjs/Subject";
import {VideoService} from "../../services/video.service";

@Component({
  selector: "app-text-cont",
  templateUrl: "./text.component.html",
  styleUrls: ["./text.component.css"]
})
export class TextComponent implements OnInit, OnDestroy {

  public text: any;
  public dataShow: string;
  public originalText: string;
  public timestamp: number;
  public editing = false;
  public locale: any = Locale.getLocale();
  private subscriptions: Array<any> = [];
  private timelineTimeSubject: Subject<any> = new Subject<any>();

  constructor(private afService: AngularfireService, private videoSvc: VideoService) {
    this.timelineTimeSubject = this.videoSvc.getFindItemInTimeline();
  }

  ngOnInit() {
    console.log("init text-cont: " + this.text);
    if (!this.text) {
      return;
    }

    // text
    const urlSub = this.afService.getAbstractData(this.text["id"]).subscribe((data): any => {
      console.log(data);
      if (data && data["text"]) {
        this.dataShow = data["text"];
      }

    });

    // cas textu
    const textInfoSub = this.afService.getDataWrapper(this.text["id"]).subscribe((data): any => {
      if (data) {
        this.timestamp = Number.parseInt(data["timestamp"]);
      }
    });
    this.subscriptions.push(textInfoSub);
    this.subscriptions.push(urlSub);
  }

  /**
   * nastaveni casu timeline na cas textu
   * @param time
   */
  public findInTimeline(time: any): void {
    this.timelineTimeSubject.next(time);
  }

  public edit(): void {
    this.editing = true;
    this.originalText = this.dataShow;
  }

  public save(): void {
    console.log("save");
    this.afService.updateText(this.text["id"], this.dataShow);
    this.editing = false;
  }

  public cancel(): void {
    console.log("cancel");
    this.dataShow = this.originalText;
    this.editing = false;
  }

  ngOnDestroy(): void {
    try {
      this.subscriptions.forEach(sub => {
        sub.unsubscribe();
      });
    } catch (e) {
      console.log(e);
    }
  }
}
