import {Component, ElementRef, OnDestroy, OnInit, ViewChild} from "@angular/core";
import {User} from "../../model/User";
import {AngularfireService} from "../../services/angularfire.service";
import {UserService} from "../../services/user.service";
import {Router} from "@angular/router";
import {Subscription} from "rxjs/Subscription";
import {VideoService} from "../../services/video.service";
import {Subject} from "rxjs/Subject";
import {Locale} from "../../constants/Locale";
import {TimelineService} from "../../services/timeline.service";

declare const vis: any;

@Component({
  selector: "app-timeline-container",
  templateUrl: "./timeline.component.html",
  styleUrls: ["./timeline.component.css"]
})
export class TimelineComponent implements OnInit, OnDestroy {

  public isPlaying = false;
  public locale: any = Locale.getLocale();
  public currentTime: number = new Date().getTime();
  public speedRate = 1;
  @ViewChild("timelineBot")
  private timelineDiv: ElementRef;
  private timeline: any;
  private user: User;
  private inst: any;
  private test: any;
  private testStart: number = new Date().getTime();
  private testEnd = 0;
  private timelineData = [];
  private subscriptions: Array<any> = [];
  private timelineTimeSubject: Subject<any> = new Subject<any>();
  private openFileSub: Subject<any> = new Subject<any>();
  private frame = 100;
  private timmerActive = false;

  constructor(private afService: AngularfireService,
              private userSvc: UserService,
              private videoSvc: VideoService,
              private tlSvc: TimelineService,
              private router: Router) {
  }

  ngOnInit(): void {
    // subject pro sdileni dat o timeline
    this.timelineTimeSubject = this.videoSvc.getTimelineSubAsVar();

    // subject pro otevreni vybraneho souboru v jinem okne
    this.openFileSub = this.tlSvc.getOpenFileSub();

    // pokud v jinem okne chci nalezt soubor na timeline, tak se zavola funkce findInTimeline(),
    // v ni se zavola funkce next() a ta zavola tenhle subscribe a nastavi cas timeline na cas toho souboru
    this.videoSvc.getFindItemInTimeline().subscribe((timestamp: any) => {
      this.currentTime = timestamp;
      this.timeline.setCustomTime(this.currentTime, "mainBar");
      this.pause();
    });
    this.init(this.inst);
  }

  public async init(instance: any) {

    this.user = this.userSvc.user;
    if (this.user === undefined) {
      this.router.navigate(["login"]);
      return;
    }

    // vyvoreni a nastaveni timeline
    try {
      if (!this.timeline) {

        this.currentTime = this.testStart;
        this.timeline = new vis.Timeline(this.timelineDiv.nativeElement, this.timelineData, {autoResize: true});
        this.timeline.addCustomTime(this.currentTime, "mainBar");
        // propagace nastaveni timeline do jinych oken (kdo si vyzadal)
        this.timelineTimeSubject.next({
          time: this.currentTime,
          play: false,
          speed: this.speedRate,
          instId: this.inst.info.id
        });
        this.timeline.on("click", (prop) => {
          this.onClick(prop);
        });
        this.timeline.on("select", (prop) => {
          this.onSelect(prop);
        });
        this.timeline.on("doubleClick", (prop) => {
          this.onDblClick(prop);
        });
        this.timeline.on("timechange", (prop) => {
          this.currentTime = new Date(prop.time).getTime();
          this.pause();
        });
      }
    } catch (e) {
      console.log(e);
    }
    console.log(this.testStart);

    // ziskani instanci
    const testData = this.afService.getInstanceData(this.test.id, instance.info.id);
    const subscription = testData.subscribe(instDataArray => {
      console.log(instDataArray);
      if (instDataArray.length === 0) {
        return;
      }
      instDataArray.forEach((item: any) => {
        this.timelineData = [];
        // data instance
        const itemWrapSub = this.afService.getDataWrapper(item.id).subscribe((itemData): any => {
          if (!itemData) {
            return;
          }
          // puntik na timeline
          const timelineItem = {
            content: "",
            title: itemData.name,
            id: itemData.id,
            start: Number.parseInt(itemData.timestamp),
            end: Number.parseInt(itemData.timestamp),
            typ: itemData.metadata.type,
            type: "point",
            inst: instance.info.id
          };

          // LOG --------------------------------
          if (itemData.metadata.type === "LOG") {

            // marker ----------------------------------------
            const markersSub = this.afService.getObject(itemData.data).subscribe((markersData): any => {
              if (!markersData) {
                return;
              }
              const markers = markersData.data;
              for (const key in markers) {
                if (!markers.hasOwnProperty(key)) {
                  continue;
                }
                const marker = markers[key];
                this.pushToTimeline({
                  id: marker.id,
                  log: {id: itemData.id, name: itemData.name, type: "LOG"},
                  title: marker["marker"],
                  start: Number.parseInt(marker["timestamp"]),
                  typ: "MARKER",
                  type: "point",
                  inst: instance.info.id
                });
              }
            });
            this.subscriptions.push(markersSub);
          }


          // VIDEO --------------------------------
          if (itemData.metadata.type === "VIDEO") {
            const videoData = this.afService.getObject(itemData.data);
            const videoSub = videoData.subscribe(data => {
              // vytvorim video html element, aby se video nacetlo, zistilo se jeho DELKA a pak se zase smaze
              const video: any = document.createElement("video");
              video.id = timelineItem["id"];
              if (data && data["video"]) {
                video.src = data["video"]["downloadUrl"];
              }
              video.addEventListener("durationchange", (event) => {
                const time: number = Math.round(timelineItem["start"] + 1000 * video.duration);
                timelineItem["end"] = time;
                timelineItem["type"] = "range";
                timelineItem["content"] = timelineItem["title"];

                video.remove();
                this.testEnd = timelineItem.end > this.testEnd ? timelineItem.end : this.testEnd;
                this.pushToTimeline(timelineItem);
              });
            });
            this.subscriptions.push(videoSub);
          }

          this.testEnd = timelineItem.end > this.testEnd ? timelineItem.end : this.testEnd;
          if (timelineItem.start < this.testStart) {
            this.testStart = timelineItem.start;
            this.currentTime = this.testStart;
            this.timeline.setCustomTime(this.currentTime, "mainBar");
            this.timelineTimeSubject.next({
              time: this.currentTime,
              play: false,
              speed: this.speedRate,
              instId: this.inst.info.id
            });
          }

          this.pushToTimeline(timelineItem);
        });
        this.subscriptions.push(itemWrapSub);
      });
    });
    this.subscriptions.push(subscription);
  }

  /**
   * vlozeni objektu do timeline
   * @param obj
   */
  private pushToTimeline(obj: any): void {
    // kontrola existence prvku
    const item: any = this.timelineData.find(x => x.id === obj.id);
    // pokud item uz je v timeline, aktualizuji se udaje
    if (item) {
      item.title = obj.title;
      item.content = obj.content;
      item.start = obj.start;
      item.end = obj.end;
      item.typ = obj.typ;
      item.type = obj.type;
    } else {
      this.timelineData.push(obj);
    }
    try {
      if (this.timeline) {
        this.timeline.setItems(this.timelineData);
        this.timeline.fit({animation: false});
      }
    } catch (err) {
      console.log(err);
    }
  }

  /**
   * tato funkce se vola kazdych +-100ms, aktualizuje cas v casove ose
   * pomoci zpetne vazby se kalibruje cas
   */
  private updateTime(): void {
    if (!this.currentTime || !this.timmerActive) {
      this.pause();
      return;
    }
    // vezmu rozdil oproti casu videii z minuleho tiku, kvuli kalibraci casu
    let diff = this.videoSvc.videoOffset;

    // velke rozdily se neberou, aby nedochazelo k cik-caku, velke rozdily se vyrovnaji u videa (pretoci se)
    diff = diff > 200 ? 0 : diff;
    diff = diff < -200 ? 0 : diff;
    if (this.timeline) {
      this.currentTime += this.frame * this.speedRate;
      this.timelineTimeSubject.next({
        time: this.currentTime,
        play: true,
        speed: this.speedRate,
        instId: this.inst.info.id
      });
      this.timeline.setCustomTime(this.currentTime, "mainBar");
    }
    // zmenseni hodnoty diff, aby nedoslo k cik-cak efektu
    setTimeout(() => {
      this.updateTime();
    }, (this.frame + 0.8 * diff));
  }

  /**
   * spusteni prehravani casove osy
   */
  public play(): void {
    this.timmerActive = true;
    setTimeout(() => {
      this.updateTime();
    }, (500));
    this.isPlaying = true;
  }

  /**
   * zastaveni prehravani casove osy
   */
  public pause(): void {
    this.timmerActive = false;
    this.timelineTimeSubject.next({
      time: this.currentTime,
      play: false,
      speed: this.speedRate,
      instId: this.inst.info.id
    });
    this.timeline.setCustomTime(this.currentTime, "mainBar");
    this.isPlaying = false;
  }

  /**
   * roztazeni timeline aby byla videt vsechna data
   */
  public fit(): void {
    if (this.timeline) {
      this.timeline.redraw();
      this.timeline.fit();
    }
  }

  /**
   * skok o urcity pocet sekund dopredu/dozadu
   * @param sec
   */
  public jump(sec: number): void {
    if (this.timeline) {
      this.currentTime += sec * 1000;
      this.timeline.setCustomTime(this.currentTime, "mainBar");
      this.timelineTimeSubject.next({
        time: this.currentTime,
        play: this.isPlaying,
        speed: this.speedRate,
        instId: this.inst.info.id
      });
    }
  }

  /**
   * na zacatek timeline
   */
  public start(): void {
    this.currentTime = this.testStart;
    this.timeline.setCustomTime(this.currentTime, "mainBar");
    this.pause();
  }

  /**
   * na konec timeline
   */
  public end(): void {
    this.currentTime = this.testEnd;
    this.timeline.setCustomTime(this.currentTime, "mainBar");
    this.pause();
  }

  public stop(): void {
    this.speedRate = 1;
    this.currentTime = this.testStart;
    this.timeline.setCustomTime(this.currentTime, "mainBar");
    this.pause();
  }

  /**
   * zrychleni/zpomaleni
   * @param rate
   */
  public fast(rate: number): void {
    let sr = this.speedRate * rate;
    sr = sr <= 0.25 ? 0.25 : (sr >= 2 ? 2 : sr);
    this.speedRate = sr;
  }

  public redraw(): void {
    console.log("red");
    this.timeline.redraw();
  }

  onSelect(properties) {
    console.log("select" + properties);
  }

  onClick(properties) {
  }

  /**
   * skok na cas, kde se kliklo, pokud je tam soubor, otevre se
   * @param properties
   */
  onDblClick(properties) {
    const item = this.timelineData.find(x => x.id === properties.item);
    if (item) {
      this.currentTime = item.start;
      this.openFileSub.next({item: item, inst: this.inst});
    } else {
      this.currentTime = new Date(properties.time).getTime();
    }
    this.pause();
  }


  ngOnDestroy(): void {
    if (this.timeline) {
      this.timeline.destroy();
    }
    console.log(this.subscriptions.length);
    this.subscriptions.forEach(sub => {
      sub.unsubscribe();
    });
  }
}
