import {Component, ElementRef, OnDestroy, OnInit} from "@angular/core";
import {AngularfireService} from "../../services/angularfire.service";
import {Locale} from "../../constants/Locale";
import {Subject} from "rxjs/Subject";
import {VideoService} from "../../services/video.service";

@Component({
  selector: "app-photo",
  templateUrl: "./video.component.html",
  styleUrls: ["./video.component.css"]
})
export class VideoComponent implements OnInit, OnDestroy {

  public style: object = {};
  public locale: any = Locale.getLocale();
  public video: any;
  public previewUrl: string;
  private timestamp: number;
  private subscriptions: Array<any> = [];
  private timelineTimeSubject: Subject<any> = new Subject<any>();


  constructor(private afService: AngularfireService, private videoSvc: VideoService) {
    this.timelineTimeSubject = this.videoSvc.getFindItemInTimeline();
  }


  ngOnInit() {
    console.log(this.video);
    const url = this.afService.getAbstractData(this.video["id"]);
    const urlSub = url.subscribe((data): any => {
      console.log(data);
      if (data && data["video"] && data["video"]["downloadUrl"]) {
        this.video["url"] = data["video"]["downloadUrl"];
        this.timestamp = Number.parseInt(data["timestamp"]);
        const vid: any = document.getElementById(this.video.id);
        this.previewUrl = "";
        if (vid) {
          vid.load();
        }
      } else if (data && data["preview"] && data["preview"]["downloadUrl"]) {
        // pokud neni hotove videa, tak muzou byt k dispozici preview fotky, zobrazim je
        this.previewUrl = data["preview"]["downloadUrl"];
      }
    });
    this.subscriptions.push(urlSub);
  }

  /**
   * nastavi cas timeline na cas fotky
   */
  public findInTimeline(): void {
    this.timelineTimeSubject.next(this.timestamp);
  }

  /**
   * zvetseni/zmenseni videa/preview fotky
   * @param pixels
   * @param element
   * @param isVideo
   */
  public zoom(pixels: number, element: ElementRef, isVideo: boolean = true): void {
    let w: any = element["style"]["width"];
    w = w.substring(0, w.length - 2);
    w = Number.parseInt(w);
    w += pixels;
    if (isVideo) {
      element["style"]["height"] = w + "px";
    }
    element["style"]["width"] = w + "px";
  }
  ngOnDestroy(): void {
    try {
      this.subscriptions.forEach(sub => {
        sub.unsubscribe();
      });
    } catch (e) {
      console.log(e);
    }
  }
}
