import {Injectable} from "@angular/core";
import * as firebase from "firebase";
import {VideoService} from "./video.service";
import {AngularFireAuth} from "angularfire2/auth";
import {AngularFireDatabase} from "angularfire2/database";
import {Observable} from "rxjs/Observable";
import {AlertComponent} from "../app/alert/alert.component";
import {Locale} from "../constants/Locale";
import {AngularFireStorage} from "angularfire2/storage";
import {TestService} from "./test.service";
import {Router} from "@angular/router";

@Injectable()
export class AngularfireService {

  public messages: any;
  public users: any;
  public abstract_data: any;
  public userTestsData: any;
  public displayName: string;
  public email: string;
  public timestamp: any;
  public userId: string;
  private locale: any = Locale.getLocale();


  constructor(private afAuth: AngularFireAuth,
              private af: AngularFireDatabase,
              private testSvc: TestService,
              private storage: AngularFireStorage,
              private router: Router,
              private videoSvc: VideoService) {
    // this.timestamp = firebase['ServerValue']['TIMESTAMP'];
    // this.messages = this.af.list('messages');
    this.users = this.af.list("/users");
    this.abstract_data = this.af.list("/abstract_data");
    this.afAuth.authState.subscribe(data => {
      this.userId = data.uid;
      console.log(this.userId);
    });
  }

  public delMember(testId: string, user: any) {
    console.log("del start");

    if (user.id == this.userId) {
      AlertComponent.error("Cannot edit.");
      return;
    }
    this.af.object(`/tests/${testId}/members/${user.id}`).remove();

    this.af.object(`/tests/${testId}/info/archived`).valueChanges().first().subscribe(value => {
      console.log("del archive");

      let status = "active";
      if (value == true) {
        status = "archived";
        const key = "archived_" + user.id;
        this.af.object(`/tests/${testId}/links/${key}`).remove();
      } else {
        const key = "active_" + user.id;
        this.af.object(`/tests/${testId}/links/${key}`).remove();
      }

      this.af.object(`/user_tests/${user.id}/${status}/${testId}`).remove();
    });

    this.af.object(`/tests/${testId}`).valueChanges().first().subscribe((test: any) => {

      console.log("del");

      const data = test.data ? Object.keys(test.data) : [];
      const instances = test.instances ? Object.values(test.instances) : [];
      const testCases = test.info["test_cases"] ? Object.values(test.info["test_cases"]) : [];
      const pre = test.info["pre_test_questionnaire"];
      const post = test.info["post_test_questionnaire"];

      if (pre) {
        this.af.object(`/data_wrapper/${pre.id}/rights/${user.id}`).remove();
      }
      if (post) {
        this.af.object(`/data_wrapper/${post.id}/rights/${user.id}`).remove();
      }

      for (const id of data) {
        this.af.object(`/user_tests_data/${user.id}/${testId}/${id}`).remove();
        this.af.object(`/data_wrapper/${id}/rights/${user.id}`).remove();
      }

      for (const testCase of testCases) {
        this.af.object(`/data_wrapper/${testCase.id}/rights/${user.id}`).remove();
      }

      for (const inst of instances) {
        this.af.object(`/test_instances/${testId}/${inst.id}`).valueChanges().first().subscribe((inst: any) => {

          const testCaseI = inst["test_case"];
          const preI = inst["pre_test_questionnaire"];
          const postI = inst["post_test_questionnaire"];

          if (inst.data) {
            const dataI = Object.keys(inst.data);
            for (const id of dataI) {
              this.af.object(`/instance_user_data/${user.id}/${inst.id}/${id}`).remove();
              this.af.object(`/data_wrapper/${id}/rights/${user.id}`).remove();
            }
          }
          if (testCaseI) {
            this.af.object(`/data_wrapper/${testCaseI.id}/rights/${user.id}`).remove();
          }
          if (preI) {
            this.af.object(`/data_wrapper/${preI.id}/rights/${user.id}`).remove();
          }

          if (postI) {
            this.af.object(`/data_wrapper/${postI.id}/rights/${user.id}`).remove();
          }
        });
      }
    });
    console.log("mazu tests konec");
    console.log("konec del");
  }

  public addMember(testId: string, user: any, type: string) {
    console.log("member start");

    // pridani do members testu
    this.af.object(`/tests/${testId}/members/${user.id}`).set({
      email: user.email,
      id: user.id,
      role: type
    });

    this.af.object(`/tests/${testId}/info/archived`).valueChanges().first().subscribe(value => {
      console.log("member archive");

      let status = "active";
      if (value == true) {
        status = "archived";
        const key = "archived_" + user.id;
        this.af.object(`/tests/${testId}/links`).update({
          [key]: `/user_tests/${user.id}/archived/${testId}`
        });
      } else {
        const key = "active_" + user.id;
        this.af.object(`/tests/${testId}/links`).update({
          [key]: `/user_tests/${user.id}/active/${testId}`
        });

        //  user_tests link
        this.af.object(`/user_tests/${this.userId}/${status}/${testId}`).valueChanges().first().subscribe((data: any) => {
          if (data) {
            this.af.object(`/user_tests/${user.id}/${status}/${testId}`).update({
              id: data.id,
              name: data.name,
              role: type,
              timestamp: data.timestamp
            });
          }
        });
      }

      this.af.object(`/tests/${testId}`).valueChanges().first().subscribe((test: any) => {
        console.log("member");

        const data = test.data ? Object.keys(test.data) : [];
        const instances = test.instances ? Object.values(test.instances) : [];
        const testCases = test.info["test_cases"] ? Object.values(test.info["test_cases"]) : [];
        const pre = test.info["pre_test_questionnaire"];
        const post = test.info["post_test_questionnaire"];

        if (pre) {
          this.af.object(`/data_wrapper/${pre.id}/rights`).update({
            [user.id]: "write"
          });
        }
        if (post) {
          this.af.object(`/data_wrapper/${post.id}/rights`).update({
            [user.id]: "write"
          });
        }

        for (const testCase of testCases) {
          this.af.object(`/data_wrapper/${testCase.id}/rights`).update({
            [user.id]: "write"
          });
        }

        for (const id of data) {
          this.af.object(`/user_tests_data/${user.id}/${testId}/${id}`).remove();
          this.af.object(`/data_wrapper/${id}/rights/${user.id}`).remove();
        }


        for (const inst of instances) {
          this.af.object(`/test_instances/${testId}/${inst.id}`).valueChanges().first().subscribe((inst: any) => {

            const testCaseI = inst["test_case"];
            const preI = inst["pre_test_questionnaire"];
            const postI = inst["post_test_questionnaire"];

            if (inst.data) {
              const dataI = Object.keys(inst.data);
              for (const id of dataI) {
                this.af.object(`/instance_user_data/${user.id}/${inst.id}/${id}`).remove();
                this.af.object(`/data_wrapper/${id}/rights/${user.id}`).remove();
              }
            }
            if (testCaseI) {
              this.af.object(`/data_wrapper/${testCaseI.id}/rights/${user.id}`).remove();
            }
            if (preI) {
              this.af.object(`/data_wrapper/${preI.id}/rights/${user.id}`).remove();
            }

            if (postI) {
              this.af.object(`/data_wrapper/${postI.id}/rights/${user.id}`).remove();
            }
          });
        }
      });
    });
  }

  public addOwner(testId: string, user: any, type) {
    console.log("owner start");

    if (user.id == this.userId) {
      AlertComponent.error("Cannot edit.");
      return;
    }
    if (type == "member") {
      this.addMember(testId, user, type);
      return;

    }
    // pridani do members testu
    this.af.object(`/tests/${testId}/members/${user.id}`).set({
      email: user.email,
      id: user.id,
      role: type
    });

    // pridani linku do testu
    this.af.object(`/tests/${testId}/info/archived`).valueChanges().first().subscribe(value => {
      console.log("owner archive");

      let status = "active";
      if (value == true) {
        status = "archived";
        const key = "archived_" + user.id;
        this.af.object(`/tests/${testId}/links`).update({
          [key]: `/user_tests/${user.id}/archived/${testId}`
        });
      } else {
        const key = "active_" + user.id;
        this.af.object(`/tests/${testId}/links`).update({
          [key]: `/user_tests/${user.id}/active/${testId}`
        });
      }

      //  user_tests link
      this.af.object(`/user_tests/${this.userId}/${status}/${testId}`).valueChanges().first().subscribe((data: any) => {
        if (data) {
          this.af.object(`/user_tests/${user.id}/${status}/${testId}`).update({
            id: data.id,
            name: data.name,
            role: type,
            timestamp: data.timestamp
          });
        }
      });
    });

    // user_test_data
    this.af.object(`/user_tests_data/${this.userId}/${testId}`).valueChanges().first().subscribe(data => {
      if (data) {
        this.af.object(`/user_tests_data/${user.id}/${testId}`).set(data);
      }
    });

    // tests link
    this.af.object(`/tests/${testId}`).valueChanges().first().subscribe((test: any) => {
      console.log("owner");
      const data = test.data ? Object.keys(test.data) : [];
      const instances = test.instances ? Object.values(test.instances) : [];
      const testCases = test.info["test_cases"] ? Object.values(test.info["test_cases"]) : [];
      const pre = test.info["pre_test_questionnaire"];
      const post = test.info["post_test_questionnaire"];

      if (pre) {
        this.af.object(`/data_wrapper/${pre.id}/rights`).update({
          [user.id]: "write"
        });
      }
      if (post) {
        this.af.object(`/data_wrapper/${post.id}/rights`).update({
          [user.id]: "write"
        });
      }

      for (const testCase of testCases) {
        this.af.object(`/data_wrapper/${testCase.id}/rights`).update({
          [user.id]: "write"
        });
      }

      for (const id of data) {
        this.af.object(`/data_wrapper/${id}/rights`).update({
          [user.id]: "write"
        });
      }



      for (const inst of instances) {
        this.af.object(`/test_instances/${testId}/${inst.id}`).valueChanges().first().subscribe((inst: any) => {

          const testCaseI = inst["test_case"];
          const preI = inst["pre_test_questionnaire"];
          const postI = inst["post_test_questionnaire"];

          if (inst.data) {
            const dataI = Object.keys(inst.data);
            // instance_user_data
            this.af.object(`/instance_user_data/${user.id}/${inst.id}`).update(inst.data);
            for (const id of dataI) {
              this.af.object(`/data_wrapper/${id}/rights`).update({
                [user.id]: "write"
              });
            }
          }
          if (testCaseI) {
            this.af.object(`/data_wrapper/${testCaseI.id}/rights`).update({
              [user.id]: "write"
            });
          }
          if (preI) {
            this.af.object(`/data_wrapper/${preI.id}/rights`).update({
              [user.id]: "write"
            });
          }

          if (postI) {
            this.af.object(`/data_wrapper/${postI.id}/rights`).update({
              [user.id]: "write"
            });
          }
        });
      }
    });
  }

  public uploadImg(file: File, test: any, instId: string) {
    if (!this.checkUser()) {
      return;
    }

    const time = this.videoSvc.getCurrentTime();
    const date = new Date(time);
    const id = this.af.createPushId();
    const name = `IMG_${date.getFullYear()}_${date.getMonth() + 1}_${date.getDate()}_${date.getHours()}_${date.getMinutes()}_${date.getSeconds()}.jpg`;

    const alert = AlertComponent.loading("Loading...");

    if (instId == undefined || instId == "") {
      this.createFile(test, "Untitled Photo", "PHOTO", id).then(() => {
        this.af.object(`/abstract_data/${id}`).set({
          photo_link: {
            downloadUrl: "",
            fileName: name,
            id: "photo_link",
            progress: 0,
            status: "PENDING"
          },
          timestamp: time + ""
        }).catch(error => {
          alert.close();
          AlertComponent.error(error);
        });
      });
    } else {
      this.createFileInst(test, instId, "Untitled Photo", "PHOTO", id).then(() => {
        this.af.object(`/abstract_data/${id}`).set({
          photo_link: {
            downloadUrl: "",
            fileName: name,
            id: "photo_link",
            progress: 0,
            status: "PENDING"
          },
          timestamp: time + ""
        }).catch(error => {
          alert.close();
          AlertComponent.error(error);
        });
      });
    }

    const filePath = "/" + id + "/" + name;
    const customMetadata = {[this.userId]: "write"};
    const task = this.storage.upload(filePath, file, {customMetadata});
    const downloadURL = task.downloadURL().subscribe(data => {
      alert.close();
      this.af.object(`/abstract_data/${id}/photo_link`).set({
        downloadUrl: data,
        fileName: name,
        id: "photo_link",
        progress: 100,
        status: "FINISHED"
      }).catch(error => {
        AlertComponent.error(error);
      });
    });
  }

  public uploadVideo(file: File, test: any, instId: string) {
    if (!this.checkUser()) {
      return;
    }
    const alert = AlertComponent.loading("Loading...");
    const time = this.videoSvc.getCurrentTime();
    const date = new Date(time);
    const id = this.af.createPushId();
    const name = `VID_${date.getFullYear()}_${date.getMonth() + 1}_${date.getDate()}_${date.getHours()}_${date.getMinutes()}_${date.getSeconds()}.mp4`;

    if (instId == undefined || instId == "") {
      this.createFile(test, "Untitled Video", "VIDEO", id).then(() => {
        this.af.object(`/abstract_data/${id}`).set({
          video: {
            downloadUrl: "",
            fileName: name,
            id: "video",
            progress: 0,
            status: "PENDING"
          },
          state: "RECORDED",
          timestamp: time + ""
        }).catch(error => {
          alert.close();
          AlertComponent.error(error);
        });
      });
    } else {
      this.createFileInst(test, instId, "Untitled Video", "VIDEO", id).then(() => {
        this.af.object(`/abstract_data/${id}`).set({
          video: {
            downloadUrl: "",
            fileName: name,
            id: "video",
            progress: 0,
            status: "PENDING"
          },
          state: "RECORDED",
          timestamp: time + ""
        }).catch(error => {
          alert.close();
          AlertComponent.error(error);
        });
      });
    }

    const filePath = "/" + id + "/" + name;
    const customMetadata = {[this.userId]: "write"};
    const task = this.storage.upload(filePath, file, {customMetadata});

    const downloadURL = task.downloadURL().subscribe(data => {
      alert.close();
      this.af.object(`/abstract_data/${id}/video`).set({
        downloadUrl: data,
        fileName: name,
        id: "video",
        progress: 100,
        status: "FINISHED"
      }).catch(error => {
        AlertComponent.error(error);
      });
    });
  }

  public addCase(testId: string, text: string): Promise<any> {
    const id: string = this.af.createPushId();
    this.af.object(`/tests/${testId}/info/test_cases/${id}`).set({
      id: id,
      name: text,
      type: "TEST_CASE"
    });

    return this.af.object(`/data_wrapper/${id}`).set({
      data: "/abstract_data/" + id,
      id: id,
      name: text,
      initialized: true,
      rights: this.setMembersRightsObject("write"),
      metadata: {schema: "json-schema", type: "TEST_CASE"},
      links: {
        [testId]: `/tests/${testId}/info/test_cases/${id}`
      },
      timestamp: new Date().getTime() + ""
    }).catch(e => {
      AlertComponent.error(e);
      console.log(e);
    });
  }

  public editCase(testId: string, caseId: string, text: string): Promise<any> {

    return this.af.object(`/data_wrapper/${caseId}`).update({
      name: text,
    }).then(() => {
      this.af.object(`/tests/${testId}/info/test_cases/${caseId}`).update({
        name: text,
      }).catch(e => {
        AlertComponent.error(e);
        console.log(e);
      });
    }).catch(e => {
      AlertComponent.error(e);
      console.log(e);
    });
  }

  public delCase(testId: string, caseId: string): Promise<any> {

    this.af.object(`/abstract_data/${caseId}`).remove().catch(e => {
      AlertComponent.error(e);
      console.log(e);
    });

    return this.af.object(`/data_wrapper/${caseId}`).remove()
      .then(() => {
        this.af.object(`/tests/${testId}/info/test_cases/${caseId}`).remove().catch(e => {
          AlertComponent.error(e);
          console.log(e);
        });
      }).catch(e => {
        AlertComponent.error(e);
        console.log(e);
      });
  }

  public addTask(taskId: string, order: number, text: string): Promise<any> {
    const id: string = this.af.createPushId();

    return this.af.object(`/abstract_data/${taskId}/${id}`).set({
      id: id,
      order: order,
      text: text,
    }).catch(e => {
      AlertComponent.error(e);
      console.log(e);
    });
  }

  public editTask(taskId: string, caseId: string, text: string): Promise<any> {
    return this.af.object(`/abstract_data/${taskId}/${caseId}`).update({
      text: text,
    }).catch(e => {
      AlertComponent.error(e);
      console.log(e);
    });
  }

  public delTask(taskId: string, caseId: string): Promise<any> {
    return this.af.object(`/abstract_data/${taskId}/${caseId}`).remove().catch(e => {
      AlertComponent.error(e);
      console.log(e);
    });
  }

  public addQuest(parentId: string, len: number, text: string): Promise<any> {
    const id: string = this.af.createPushId();
    return this.af.object(`/abstract_data/${parentId}/${id}`).set({
      id: id,
      order: len,
      text: text
    }).catch(e => {
      AlertComponent.error(e);
      console.log(e);
    });
  }

  public delQuest(parentId: string, id: string): Promise<any> {
    return this.af.object(`/abstract_data/${parentId}/${id}`).remove()
      .catch(e => {
        AlertComponent.error(e);
        console.log(e);
      });
  }


  public delQuestInst(testId: string, instId: string, parentId: string, type: string): Promise<any> {
    this.af.object(`/abstract_data/${parentId}`).remove()
      .catch(e => {
        AlertComponent.error(e);
        console.log(e);
      });
    return this.af.object(`/test_instances/${testId}/${instId}/${type}`).remove().catch(e => {
      AlertComponent.error(e);
      console.log(e);
    });

  }

  public editQuest(parentId: string, id: string, text: string): Promise<any> {
    return this.af.object(`/abstract_data/${parentId}/${id}`).update({
      text: text
    }).catch(e => {
      AlertComponent.error(e);
      console.log(e);
    });
  }

  public editQuestAns(parentId: string, id: string, answer: string): Promise<any> {
    return this.af.object(`/abstract_data/${parentId}/${id}`).update({
      answer: answer
    }).catch(e => {
      AlertComponent.error(e);
      console.log(e);
    });
  }

  public updateQuestionnareNameTest(testId: string, questId: string, name: string, key: string): Promise<any> {
    return this.af.object(`/data_wrapper/${questId}`).update({
      name: name
    }).then(() => {
      this.af.object(`/tests/${testId}/info/${key}`).update({
        name: name
      });
    });
  }

  public updateQuestionnareNameInst(testId: string, instId: string, questId: string, name: string, key: string): Promise<any> {
    return this.af.object(`/data_wrapper/${questId}`).update({
      name: name
    }).then(() => {
      this.af.object(`/test_instances/${testId}/${instId}/${key}`).update({
        name: name
      });
    });
  }

  public updateQuestionTextTest(testId: string, questId: string, name: string, key: string): Promise<any> {
    return this.af.object(`/data_wrapper/${questId}`).update({
      name: name
    }).then(() => {
      this.af.object(`/tests/${testId}/info/${key}`).update({
        name: name
      });
    });
  }

  public addParticipant(testId: string, name: string): Promise<any> {
    const id = this.af.createPushId();
    return this.af.object(`/tests/${testId}/info/participants/${id}`).set({
      id: id,
      name: name
    }).catch(e => {
      AlertComponent.error(e);
      console.log(e);
    });
  }

  public editParticipant(testId: string, partId: string, name: string): Promise<any> {
    return this.af.object(`/tests/${testId}/info/participants/${partId}`).update({
      name: name
    }).catch(e => {
      AlertComponent.error(e);
      console.log(e);
    });
  }

  public delParticipant(testId: string, partId: string): Promise<any> {
    return this.af.object(`/tests/${testId}/info/participants/${partId}`).remove().catch(e => {
      AlertComponent.error(e);
      console.log(e);
    });
  }

  public addMarker(testId: string, order: number, text: string): Promise<any> {
    const id = this.af.createPushId();
    return this.af.object(`/tests/${testId}/markers/${id}`).set({
      id: id,
      order: order,
      text: text
    }).catch(e => {
      AlertComponent.error(e);
      console.log(e);
    });
  }

  public editMarker(testId: string, id: string, text: string): Promise<any> {
    return this.af.object(`/tests/${testId}/markers/${id}`).update({
      text: text
    }).catch(e => {
      AlertComponent.error(e);
      console.log(e);
    });
  }

  public delMarker(testId: string, id: string): Promise<any> {
    return this.af.object(`/tests/${testId}/markers/${id}`).remove().catch(e => {
      AlertComponent.error(e);
      console.log(e);
    });
  }

  public getTimestamp(): any {
    return this.timestamp;
  }

  public deleteData(path: string): Promise<any> {
    return this.af.list(path).remove();
  }

  public setData(path: string, item: any): any {
    const ret = this.af.list(path).push(item);
    ret.then(resp => {
      this.af.object(path + "/" + resp.key + "/id").set(resp.key);
    });
    return ret;
  }

  public editLog(path: string, log: any): any {
    return this.af.object(path).update({comment: log.comment, marker: log.marker});
  }

  public editValue(path: string, key: string, value: string): any {
    return this.af.object(path + "/" + key).set(value);
  }

  public getMarkers(testId: string) {
    return this.af.list(`/tests/${testId}/markers`).valueChanges();
  }

  public getUserTestsData(userId: string) {
    return this.af.list("/user_tests_data/" + userId).valueChanges();
  }

  public getUserSpecificTestData(userId: string, testId: string) {
    return this.af.object("/user_tests_data/" + userId + "/" + testId).valueChanges();
  }

  public getTestInstData(testId: string) {
    return this.af.object("/test_instances/" + testId).valueChanges();
  }

  public getUserTests(userId: string) {
    return this.af.object("/user_tests/" + userId).valueChanges();
  }

  public getObject(url: string): Observable<any> {
    return this.af.object(url).valueChanges();
  }

  public getList(url: string) {
    return this.af.list(url).valueChanges();
  }

  public getAbstractData(dataWrapperId: string) {
    return this.af.object("/abstract_data/" + dataWrapperId).valueChanges();
  }

  public getAbstractDataList(dataWrapperId: string) {
    return this.af.list("/abstract_data/" + dataWrapperId).valueChanges();
  }

  public getTest(testId: string) {
    return this.af.object("/tests/" + testId).valueChanges();
  }

  public getAllInstances(id: string) {
    return this.af.object("/test_instances/" + id).valueChanges();
  }

  public getTestData(testId: string) {
    return this.af.list("/tests/" + testId + "/data").valueChanges();
  }

  public getTestInfoAll(testId: string) {
    return this.af.object("/tests/" + testId + "/info").valueChanges();
  }

  public getTestTestCases(testId: string) {
    return this.af.list("/tests/" + testId + "/info/test_cases").valueChanges();
  }

  public getTestParticipants(testId: string) {
    return this.af.list("/tests/" + testId + "/info/participants").valueChanges();
  }

  public getTestMarkers(testId: string) {
    return this.af.list("/tests/" + testId + "/markers").valueChanges();
  }
  public getTestMembers(testId: string) {
    return this.af.list("/tests/" + testId + "/members").valueChanges();
  }

  public getUsers() {
    return this.af.object("/users").valueChanges();
  }

  public getInstInfoQuest(testId: string, instId: string) {
    return this.af.object(`/test_instances/${testId}/${instId}`).valueChanges();
  }

  public getInstTestCase(testId: string, instId: string) {
    return this.af.object(`/test_instances/${testId}/${instId}/test_case`).valueChanges();
  }

  public setInstTestCase(testId: string, instId: string, casse: any): Promise<any> {
    return this.af.object(`/test_instances/${testId}/${instId}/test_case`).set({
      id: casse.id,
      name: casse.name,
      type: casse.type
    }).catch((error => {
      console.log(error);
      AlertComponent.info(this.locale.updateFail);

    }));
  }

  public getTestInfo(testId: string) {
    return this.af.object("/tests/" + testId + "/info/info").valueChanges();
  }

  public getInstInfo(testId: string, instId: string) {
    return this.af.object(`/test_instances/${testId}/${instId}/notes`).valueChanges();
  }

  public getInstanceData(testId: string, instId: string) {
    return this.af.list(`/test_instances/${testId}/${instId}/data`).valueChanges();
  }

  public getInstance(testId: string, instId: string) {
    return this.af.object(`/test_instances/${testId}/${instId}`).valueChanges();
  }

  public getTestInstances(testId: string, instanceId: string) {
    return this.af.object("/test_instances/" + testId + "/" + instanceId).valueChanges();
  }

  public getDataWrapper(dataWrapperId: string): Observable<any> {
    return this.af.object("/data_wrapper/" + dataWrapperId).valueChanges();
  }

  public getDetailedData(id: string) {
    return this.af.object("/abstract_data/" + id).valueChanges();
  }

  public async updateText(id: string, value: string): Promise<any> {
    return this.af.object(`/abstract_data/${id}`).update({text: value}).then(() => {
      AlertComponent.info(this.locale.updateOk);
    }).catch((error => {
      console.log(error);
      AlertComponent.info(this.locale.updateFail);

    }));
  }

  public async updateTestInfo(testId: string, value: string): Promise<any> {
    return this.af.object("/tests/" + testId + "/info").update({info: value}).then(() => {
      AlertComponent.info(this.locale.updateOk);
    }).catch((error => {
      console.log(error);
      AlertComponent.info(this.locale.updateFail);

    }));
  }


  public async updateInstInfo(testId: string, instId: string, value: string): Promise<any> {
    return this.af.object(`/test_instances/${testId}/${instId}`).update({notes: value}).then(() => {
      AlertComponent.info(this.locale.updateOk);
    }).catch((error => {
      console.log(error);
      AlertComponent.info(this.locale.updateFail);

    }));
  }

  public renameTest(test: any): void {
    if (!this.checkUser()) {
      return;
    }
    try {
      this.af.object(`/tests/${test.id}/info`).update({name: test.name});
    } catch (e) {
      console.log("nepodarilo se prejmenovat " + test);
      console.log("nepodarilo se prejmenovat chyba" + e);
      AlertComponent.error(this.locale.renameFail);
    }
  }

  public renameInst(test: any, inst: any): void {
    if (!this.checkUser()) {
      return;
    }
    try {
      this.af.object(`/test_instances/${test.id}/${inst.info.id}`).update({name: inst.info.name});
      this.af.object(`/tests/${test.id}/instances/${inst.info.id}`).update({name: inst.info.name});
    } catch (e) {
      console.log("nepodarilo se prejmenovat " + inst);
      console.log("nepodarilo se prejmenovat chyba" + e);
      AlertComponent.error(this.locale.renameFail);
    }
  }

  public renameFileInst(test: any, inst: any, item: any): void {
    if (!this.checkUser()) {
      return;
    }
    try {
      this.af.object(`/data_wrapper/${item.id}`).update({name: item.name});
      this.af.object(`/instance_user_data/${this.userId}/${inst.info.id}/${item.id}`).update({name: item.name});
      this.af.object(`/test_instances/${test.id}/${inst.info.id}/data/${item.id}`).update({name: item.name});
    } catch (e) {
      console.log("nepodarilo se prejmenovat " + item);
      console.log("nepodarilo se prejmenovat chyba" + e);
      AlertComponent.error(this.locale.renameFail);
    }
  }

  public renameFileTest(test: any, item: any): void {
    if (!this.checkUser()) {
      return;
    }
    try {
      this.af.object(`/data_wrapper/${item.id}`).update({name: item.name});
      this.af.object(`/user_tests_data/${this.userId}/${test.id}/${item.id}`).update({name: item.name});
      this.af.object(`/tests/${test.id}/data/${item.id}`).update({name: item.name});
    } catch (e) {
      console.log("nepodarilo se prejmenovat " + item);
      console.log("nepodarilo se prejmenovat chyba" + e);
      AlertComponent.error(this.locale.renameFail);
    }
  }

  /**
   * Smazani souboru
   * @param testId id testu, kde je soubor ulozen
   * @param itemId id souboru
   */
  public removeItem(testId: string, item: any) {
    if (!this.userId) {
      AlertComponent.error("User not logged");
      return;
    }
    console.log(item);

    return this.af.object(`/user_tests_data/${this.userId}/${testId}/${item.id}`).remove().then(() => {
      console.log("removed from utd");
      this.af.object(`/tests/${testId}/data/${item.id}`).remove().then(() => {
        console.log("removed from test");
        this.af.object(`/abstract_data/${item.id}`).remove().then(() => {
          console.log("removed from abstract");
          this.af.object(`/data_wrapper/${item.id}`).remove().then(() => {
            console.log("Item removed");
            if (item.type == "PHOTO" || item.type == "VIDEO") {
              this.storage.ref(`/${item.id}/${item.fileName}`).delete();
            }
          });
        });
      });
    });
  }

  public removeItemInst(testId: string, instId: string, itemId: string, type: string) {
    if (!this.checkUser()) {
      return;
    }
    return this.af.object(`/instance_user_data/${this.userId}/${instId}/${itemId}`).remove().then(() => {
      console.log("removed from utd");
      this.af.object(`/test_instances/${testId}/${instId}/data/${itemId}`).remove().then(() => {
        console.log("removed from test");
        this.af.object(`/abstract_data/${itemId}`).remove().then(() => {
          console.log("removed from abstract");
          this.af.object(`/data_wrapper/${itemId}`).remove().then(() => {
            console.log("Item removed");
          });
        });
      });
    });
  }

  public createQuest(testId: string, type: string): Promise<any> {
    // pokud nemam userId (nejsem prihlaseny) return
    if (!this.checkUser()) {
      return undefined;
    }
    const id: string = this.af.createPushId();
    const name: string = (type == "post_test_questionnaire") ? "Post-test Questionnaire" : "Pre-test Questionnaire";

    this.af.object(`/tests/${testId}/info/${type}`).set({
      id: id,
      name: name,
      type: "QUESTIONNAIRE"
    }).catch(error => {
      AlertComponent.error(error);
    });

    return this.af.object(`/data_wrapper/${id}`).set({
      data: "/abstract_data/" + id,
      id: id,
      name: name,
      initialized: true,
      rights: this.setMembersRightsObject("write"),
      metadata: {schema: "json-schema", type: "QUESTIONNAIRE"},
      links: {
        [testId]: `/tests/${testId}/info/${type}`
      },
      timestamp: new Date().getTime() + ""
    }).catch(error => {
      AlertComponent.error(error);
    });
  }

  public createQuestInst(testId: string, instId: string, type: string): Promise<any> {
    // pokud nemam userId (nejsem prihlaseny) return
    if (!this.checkUser()) {
      return undefined;
    }
    const id: string = this.af.createPushId();
    const name: string = (type == "post_test_questionnaire") ? "Post-test Questionnaire" : "Pre-test Questionnaire";

    this.af.object(`/test_instances/${testId}/${instId}/${type}`).set({
      id: id,
      name: name,
      type: "ANSWERABLE_QUESTIONNAIRE"
    }).catch(error => {
      AlertComponent.error(error);
    });

    // ziskam questionnare testu (chci ID)
    this.af.object(`/tests/${testId}/info/${type}`).valueChanges().subscribe((quest: any) => {
      if (quest) {
        // podle id ziskam otazky z dotazniku
        this.af.object(`/abstract_data/${quest.id}`).valueChanges().subscribe((items: any) => {
          for (const i in items) {
            items[i]["answer"] = "";
          }
          this.af.object(`/abstract_data/${id}`).set(items).catch(error => {
            AlertComponent.error(error);
          });
          
        });
      }
    });

    return this.af.object(`/data_wrapper/${id}`).set({
      data: "/abstract_data/" + id,
      id: id,
      name: name,
      initialized: true,
      rights: this.setMembersRightsObject("write"),
      metadata: {schema: "json-schema", type: "QUESTIONNAIRE"},
      links: {
        [instId]: `/test_instances/${testId}/${instId}/${type}`
      },
      timestamp: new Date().getTime() + ""
    }).catch(error => {
      AlertComponent.error(error);
    });
  }

  public deleteQuestAll(questId: string, testId: string, type: string): Promise<any> {
    // pokud nemam userId (nejsem prihlaseny) return
    if (!this.checkUser()) {
      return undefined;
    }

    this.af.object(`/abstract_data/${questId}`).remove()
      .catch(error => {
        AlertComponent.error(error);
      });
    this.af.object(`/tests/${testId}/info/${type}`).remove()
      .catch(error => {
        AlertComponent.error(error);
      });
    return this.af.object(`/data_wrapper/${questId}`).remove()
      .catch(error => {
        AlertComponent.error(error);
      });
  }

  /**
   * Vytvori soubor instance v firebase DB
   * @param test do ktereho se vytvori soubor
   * @param instId do ktere se vytvori soubor
   * @param name jmeno souboru
   * @param type typ souboru
   */
  public createFileInst(test: any, instId: any, name: string, type: string, id: string = this.af.createPushId()): Promise<void> {
    // pokud nemam userId (nejsem prihlaseny) return
    if (!this.checkUser()) {
      return undefined;
    }

    // vlozeni souboru do user_tests_data
    this.af.object(`/instance_user_data/${this.userId}/${instId}/${id}`).set({
      id: id,
      name: name,
      type: type.toUpperCase()
    }).catch(error => {
      AlertComponent.error(error);
    });

    // vlozeni souboru do tests
    this.af.object(`/test_instances/${test.id}/${instId}/data/${id}`).set({
      id: id,
      name: name,
      type: type.toUpperCase()
    }).catch(error => {
      AlertComponent.error(error);
    });

    // pokud se jedna o LOG, vztvorim zaznam v abstract_data a ulozim tam link na markery
    if (type.toUpperCase() === "LOG") {
      this.af.object(`/abstract_data/${id}`).set({
        markersLink: `/tests/${test.id}/markers`
      }).catch(error => {
        AlertComponent.error(error);
      });
    }

    // vlozeni souboru do data_wrapper
    return this.af.object(`/data_wrapper/${id}`).set({
      data: "/abstract_data/" + id,
      id: id,
      name: name,
      initialized: true,
      rights: this.setMembersRightsObject("write"),
      metadata: {schema: "json-schema", type: type.toUpperCase()},
      links: {
        [instId]: `/test_instances/${test.id}/${instId}/data/${id}`,
        [this.userId]: `/instance_user_data/${this.userId}/${instId}/${id}`
      },
      timestamp: this.videoSvc.getCurrentTime() + ""
    });
  }

  /**
   * Vytvori soubor testu v firebase DB
   * @param test do ktereho se vytvori soubor
   * @param name jmeno souboru
   * @param type typ souboru
   */
  public createFile(test: any, name: string, type: string, id: string = this.af.createPushId()): Promise<void> {
    // pokud nemam userId (nejsem prihlaseny) return
    if (!this.userId) {
      AlertComponent.error(Locale.getLocale().noUserId);
      return;
    }

    // vlozeni souboru do user_tests_data
    this.af.object(`/user_tests_data/${this.userId}/${test.id}/${id}`).set({
      id: id,
      name: name,
      type: type.toUpperCase()
    }).catch(error => {
      AlertComponent.error(error);
    });

    // vlozeni souboru do tests
    this.af.object(`/tests/${test.id}/data/${id}`).set({
      id: id,
      name: name,
      type: type.toUpperCase()
    }).catch(error => {
      AlertComponent.error(error);
    });

    // pokud se jedna o LOG, vztvorim zaznam v abstract_data a ulozim tam link na markery
    if (type.toUpperCase() === "LOG") {
      this.af.object(`/abstract_data/${id}`).set({
        markersLink: `/tests/${test.id}/markers`
      }).catch(error => {
        AlertComponent.error(error);
      });
    }
    // vlozeni souboru do data_wrapper
    return this.af.object(`/data_wrapper/${id}`).set({
      data: "/abstract_data/" + id,
      id: id,
      name: name,
      initialized: true,
      rights: this.setMembersRightsObject("write"),
      metadata: {schema: "json-schema", type: type.toUpperCase()},
      links: {
        [test.id]: `/tests/${test.id}/data/${id}`,
        [this.userId]: `/user_tests_data/${this.userId}/${test.id}/${id}`
      },
      timestamp: this.videoSvc.getCurrentTime() + ""
    }).catch(error => {
      AlertComponent.error(error);
    });
  }

  public createInst(test: any, name: string): void {
    if (!this.checkUser()) {
      return;
    }
    // vytvorim si id souboru
    const id = this.af.createPushId();

    this.af.object(`/test_instances/${test.id}/${id}`).set({
      id: id,
      name: name,
      participant: "",
      test_definition_id: test.id
    }).catch(error => {
      AlertComponent.error(error);
    });

    this.af.object(`/tests/${test.id}/instances/${id}`).set({
      id: id,
      name: name,
      test_definition_id: test.id,
      timestamp: new Date().getTime() + ""
    }).catch(error => {
      AlertComponent.error(error);
    });

  }

  /**
   * Saves a message to the Firebase Realtime Database
   * @param text
   */
  public sendMessage(text): void {
    const message = {
      message: text,
      displayName: this.displayName,
      email: this.email,
      timestamp: Date.now() + ""
    };
    this.messages.push(message);
  }

  private checkUser(): boolean {
    if (!this.userId) {
      AlertComponent.error(Locale.getLocale().noUserId);
      console.log("no user id: " + this.userId);
      return false;
    }
    return true;
  }

  /**
   * Logs in the user
   * @returns {firebase.Promise<FirebaseAuthState>}
   */
  loginWithGoogle() {
    this.afAuth.auth.signInWithPopup(new firebase.auth.GoogleAuthProvider()).then(data => {
      console.log(data);
      const user = data.user;
      if (!user) {
        AlertComponent.error("Log in failed.");
        return;
      }
      this.router.navigate(["tests"]);

      this.af.object(`/users/${user.uid}`).update({
        email: user.email,
        name: user.displayName
      });

      const email = user.email.replace(/\./g, "_");

      this.af.object(`/user_mapping`).update({
        [email]: user.uid
      });
    });
  }

  /**
   * Logs out the current user
   */
  logout() {
    console.log("logout");
    this.afAuth.auth.signOut();
  }

  public createId(): string {
    return this.af.createPushId();
  }

  private setMembersRightsObject(right: string = "write"): any {
    const members = this.testSvc.getMembers();
    const ob: any = {};
    for (const member of members) {
      ob[member.id] = right;
    }
    console.log(ob);
    return ob;
  }
}
