import {ComponentFactoryResolver, Injectable} from "@angular/core";
import {TestItem} from "../model/TestItem";
import * as GoldenLayout from "golden-layout";
import {Router} from "@angular/router";
import {TimelineVideoComponent} from "../app/timeline-video/timeline-video.component";
import {TestService} from "./test.service";
import {VideoService} from "./video.service";
import {PhotoComponent} from "../app/photo/photo.component";
import {TimelineService} from "./timeline.service";
import {VideoComponent} from "../app/video/video.component";
import {Locale} from "../constants/Locale";
import {TextComponent} from "../app/text/text.component";
import {LogsComponent} from "../app/logs/logs.component";
import {TimelineComponent} from "../app/timeline/timeline.component";
import {FilesComponent} from "../app/files/files.component";
import {InfoComponent} from "../app/info/info.component";
import {QuestComponent} from "../app/quest/quest.component";
import {QuestInstComponent} from "../app/quest-inst/quest-inst.component";
import {TestCasesComponent} from "../app/test-cases/test-cases.component";
import {TestCasesInstComponent} from "../app/test-cases-inst/test-cases-inst.component";
import {ParticipantsComponent} from "../app/participants/participants.component";
import {MarkersComponent} from "../app/markers/markers.component";
import {MembersComponent} from "../app/members/members.component";

@Injectable()
export class GoldenLayoutService {

  public layout: any;
  private test: TestItem;
  private currentTime: any;
  private locale: any = Locale.getLocale();
  // konfigurace layoutu
  private config: any = {
    content: [{
      type: "row",
      isClosable: false,
      content: [
        {
          type: "stack",
          id: "filesystem",
          width: 20,
          content: [{
            type: "component",
            isClosable: false,
            title: "Project",
            componentName: "filesystem",
            componentState: {}
          }]
        }, {
          type: "stack",
          id: "instaces",
          isClosable: false,
          title: this.locale.allFiles,
          content: [
            {
              type: "row",
              id: "allFiles",
              isClosable: false,
              title: this.locale.allFiles,
              content: [
                {
                  type: "column",
                  width: 70,
                  isClosable: false,
                  content: [
                    {
                      type: "stack",
                      id: "-videoTimeline",
                      isClosable: false,
                      title: "Main",
                      content: [
                      ]
                    }
                  ]
                },
                {
                  type: "column",
                  content: [
                    {
                      type: "stack",
                      id: "-log",
                      isClosable: false,
                      content: [
                      ]
                    }
                  ]
                }
              ]
            }
          ]
        }]
    }]
  };


  constructor(private testSvc: TestService,
              private videoSvc: VideoService,
              private tlSvc: TimelineService,
              private componentFactoryResolver: ComponentFactoryResolver) {
    // pokud otevru soubor v strome projektu, tak se zavola tato funkce
    this.tlSvc.getOpenFileSub().asObservable().subscribe((obj: any) => {
      const item = obj.item;
      const inst = obj.inst;
      let elArr = [];
      console.log("oteviram item");

      elArr = this.layout.root.getItemsById(item.id);
      if (elArr.length > 0) {
        elArr[0].parent.setActiveContentItem(elArr[0]);
      } else {
        if (item.typ === "MARKER") {
          this.showLog(item.log, inst);
        }
        if (item.typ === "VIDEO") {
          this.showVideo(item, inst);
        }
        if (item.typ === "PHOTO") {
          this.showPhoto(item, inst);
        }
        if (item.typ === "TEXT") {
          this.showText(item, inst);
        }
      }

    });
  }

  public showQuest(quest: any, testId: string): void {
    const item: any = {id: "quest", name: "Questionnaires", testId: testId, data: quest};
    this.showFile(item, "quest", null, "-videoTimeline", 50);
  }

  public showCases(cases: any, testId: string): void {
    const item: any = {id: "cases", name: "Test Cases", testId: testId, data: cases};
    this.showFile(item, "cases", null, "-videoTimeline", 50);
  }

  public showCasesInst(inst: any): void {
    const instId = !inst ? "" : inst.info.id;
    const name = !inst ? "" : inst.info.name;
    const item: any = {id: "cases-" + instId, name: name + " - Test Cases", instId: instId, testId: this.test.id};
    this.showFile(item, "casesInst", inst, "-videoTimeline", 50);
  }

  public showInfo(testId: string, inst: any): void {
    const instId = !inst ? "" : inst.info.id;
    const item: any = {id: "info", name: "Notes", testId: testId, instId: instId};
    this.showFile(item, "info", inst, "-videoTimeline", 50);
  }
  public showParticipants(testId: string): void {
    const item: any = {id: "participants", name: "Participants", testId: testId};
    this.showFile(item, "participants", null, "-videoTimeline", 50);
  }
  public showMembers(testId: string): void {
    const item: any = {id: "members", name: "Members", testId: testId};
    this.showFile(item, "members", null, "-videoTimeline", 50);
  }
  public showMarkers(testId: string): void {
    const item: any = {id: "markers", name: "Markers", testId: testId};
    this.showFile(item, "markers", null, "-videoTimeline", 50);
  }

  public showLog(item: TestItem, inst: any): void {
    this.showFile(item, "log", inst, "-log", 100);
  }

  public showText(item: TestItem, inst: any): void {
    this.showFile(item, "text", inst,  "-videoTimeline", 30);
  }

  public showVideo(item: TestItem, inst: any): void {
    this.showFile(item, "video", inst, "-videoTimeline", 50);
  }
  public showVideoTimeline(item: TestItem, inst: any): void {
    this.openInst(inst);
    this.videoSvc.getVideoSubAsVar().next({item: item, instId: inst.info.id});
    const elArr = this.layout.root.getItemsById(inst.info.id + "-selectedVideo");
    if (elArr.length > 0) {
      elArr[0].parent.setActiveContentItem(elArr[0]);
    }
  }

  public showVideoSingle(item: TestItem): void {
    this.showFile(item, "video", null, "-videoTimeline", 50);
  }

  public showPhoto(item: TestItem, inst: any): void {
    this.showFile(item, "photo", inst, "-videoTimeline", 50);
  }

  public showQuestInst(quest: any, testId: string, inst: any): void {
    const instId = !inst ? "" : inst.info.id;
    const name = !inst ? "" : inst.info.name;
    const item: any = {id: "quest-" + instId, name: name + " - Questionnaires", testId: testId, instId: instId, data: quest};
    this.showFile(item, "questInst", inst, "-videoTimeline", 50);
  }

  public openInst(inst: any) {
    // zkontorluju jestli uz neni otevrena
    const elArr = this.layout.root.getItemsById(inst.info.id);
    if (elArr.length > 0) {
      // pokud je otevrena, focus na okno
      elArr[0].parent.setActiveContentItem(elArr[0]);
    } else {
      // instance neni otevrena, otevru ji
      this.showInst(inst);

    }
  }

  /**
   * pokud je okno otevrene a chci ho otevrit znova, otevre se puvodni (asi neni vsude implementovano)
   */
  public focusItem(item: any, inst: any): void {
    if (!inst) {
      return;
    }
    const elArr = this.layout.root.getItemsById(inst.info.id);
    if (elArr.length > 0) {
      elArr[0].parent.setActiveContentItem(elArr[0]);
    }
  }

  /**
   * zobrazeni okna s daty
   * @param item
   * @param componentName
   * @param inst
   * @param containerId
   * @param height vyska okna, pokud jich je vice pod sebou
   */
  private showFile(item: any, componentName: string, inst: any, containerId: string, height: number): void {
    if (inst) {
      this.openInst(inst);
    }
    const instId = !inst ? "" : inst.info.id;
    let newItemConfig;
    console.log("jdem na to");

    let el;
    let elArr = this.layout.root.getItemsById(instId + containerId);

    // pokud jsem smazal kontejner, zalozim novej
    if (elArr.length === 0) {
      elArr = this.layout.root.getItemsById(containerId);
      if (elArr.length === 0) {
        newItemConfig = {
          type: "stack",
          id: containerId,
          height: height,
          content: [{
            id: item.id,
            type: "component",
            componentName: componentName,
            title: item["name"],
            componentState: {
              data: item
            }
          }]
        };
      } else {
        el = elArr[0];
      }
      // jinak jen pridam item
    } else {

      el = elArr[0];
      if (instId === "") {
        elArr = this.layout.root.getItemsById("allFiles");
        item["allFiles"] = true;
      } else {
        elArr = this.layout.root.getItemsById(instId);
        item["allFiles"] = false;
      }
      if (elArr.length > 0) {
        elArr[0].parent.setActiveContentItem(elArr[0]);
      }
      newItemConfig = {
        id: item.id,
        type: "component",
        componentName: componentName,
        title: item["name"],
        componentState: {
          data: item
        }
      };
    }
    el.addChild(newItemConfig);
  }

  /**
   * zobrazeni instance
   * @param inst
   */
  public showInst(inst: any) {
    console.log(inst);
    let newItemConfig;
    let container;
    const containerArr = this.layout.root.getItemsById("instaces");

    container = containerArr[0];
    newItemConfig = {
      type: "row",
      id: inst.info.id,
      isClosable: true,
      title: inst.info.name,
      content: [
        {
          type: "column",
          width: 70,
          content: [
            {
              type: "stack",
              id: inst.info.id + "-videoTimeline",
              height: 60,
              content: [{
                type: "component",
                isClosable: false,
                title: "Selected Video",
                id: inst.info.id + "-selectedVideo",
                componentName: "videoTimeline",
                componentState: {
                  instId: inst.info.id
                }
              }
              ]
            }, {
              type: "component",
              title: "Timeline",
              componentName: "timeline",
              componentState: {
                instance: inst
              }
            }]
        },
        {
          type: "column",
          content: [
            {
              type: "stack",
              id: inst.info.id + "-log",
              isClosable: false,
              content: [
              ]
            }]
        }
      ]
    };
    container.addChild(newItemConfig);
  }

  /**
   * zavre se okno podle id
   * @param id
   */
  public closeById(id: string) {
    const items = this.layout.root.getItemsById(id);
    for (const item of items) {
      item.remove();
    }
  }

  /**
   * inicializace golden layoutu
   * @param test, ktery se ma nacist
   * @param container, ve kterem se vykresli GL
   */
  public initLayout(test: TestItem, container: any, viewContainer: any): void {
    this.test = test;
    this.layout = new GoldenLayout(this.config, container.nativeElement);

    this.layout.registerComponent("filesystem", (container, componentState) => {
      const factory = this.componentFactoryResolver.resolveComponentFactory(FilesComponent);

      const compRef = viewContainer.createComponent(factory);
      // compRef.instance.setEventHub(this.layout.eventHub);
      compRef.instance.test = this.test;
      container.getElement().append(compRef.location.nativeElement);

      container["compRef"] = compRef;

      compRef.changeDetectorRef.detectChanges();
    });

    this.layout.registerComponent("videoTimeline", (container, componentState) => {
      const factory = this.componentFactoryResolver.resolveComponentFactory(TimelineVideoComponent);

      const compRef = viewContainer.createComponent(factory);
      // compRef.instance.setEventHub(this.layout.eventHub);
      container.getElement().append(compRef.location.nativeElement);
      compRef.instance.instId = componentState.instId;

      container["compRef"] = compRef;

      compRef.changeDetectorRef.detectChanges();
    });

    this.layout.registerComponent("info", (container, componentState) => {
      const factory = this.componentFactoryResolver.resolveComponentFactory(InfoComponent);

      const compRef = viewContainer.createComponent(factory);
      // compRef.instance.setEventHub(this.layout.eventHub);
      container.getElement().append(compRef.location.nativeElement);
      compRef.instance.data = componentState.data;
      compRef.instance.testId = this.test.id;

      container["compRef"] = compRef;

      compRef.changeDetectorRef.detectChanges();
    });

    this.layout.registerComponent("participants", (container, componentState) => {
      const factory = this.componentFactoryResolver.resolveComponentFactory(ParticipantsComponent);

      const compRef = viewContainer.createComponent(factory);
      // compRef.instance.setEventHub(this.layout.eventHub);
      container.getElement().append(compRef.location.nativeElement);
      compRef.instance.data = componentState.data;
      compRef.instance.testId = this.test.id;

      container["compRef"] = compRef;

      compRef.changeDetectorRef.detectChanges();
    });

    this.layout.registerComponent("quest", (container, componentState) => {
      const factory = this.componentFactoryResolver.resolveComponentFactory(QuestComponent);

      const compRef = viewContainer.createComponent(factory);
      // compRef.instance.setEventHub(this.layout.eventHub);
      container.getElement().append(compRef.location.nativeElement);
      compRef.instance.data = componentState.data;
      compRef.instance.testId = this.test.id;

      container["compRef"] = compRef;

      compRef.changeDetectorRef.detectChanges();
    });

    this.layout.registerComponent("cases", (container, componentState) => {
      const factory = this.componentFactoryResolver.resolveComponentFactory(TestCasesComponent);

      const compRef = viewContainer.createComponent(factory);
      container.getElement().append(compRef.location.nativeElement);
      compRef.instance.data = componentState.data;
      compRef.instance.testId = this.test.id;

      container["compRef"] = compRef;

      compRef.changeDetectorRef.detectChanges();
    });

    this.layout.registerComponent("casesInst", (container, componentState) => {
      const factory = this.componentFactoryResolver.resolveComponentFactory(TestCasesInstComponent);

      const compRef = viewContainer.createComponent(factory);
      container.getElement().append(compRef.location.nativeElement);
      compRef.instance.data = componentState.data;
      compRef.instance.testId = this.test.id;

      container["compRef"] = compRef;

      compRef.changeDetectorRef.detectChanges();
    });

    this.layout.registerComponent("markers", (container, componentState) => {
      const factory = this.componentFactoryResolver.resolveComponentFactory(MarkersComponent);

      const compRef = viewContainer.createComponent(factory);
      container.getElement().append(compRef.location.nativeElement);
      compRef.instance.data = componentState.data;
      compRef.instance.testId = this.test.id;

      container["compRef"] = compRef;

      compRef.changeDetectorRef.detectChanges();
    });

    this.layout.registerComponent("members", (container, componentState) => {
      const factory = this.componentFactoryResolver.resolveComponentFactory(MembersComponent);

      const compRef = viewContainer.createComponent(factory);
      container.getElement().append(compRef.location.nativeElement);
      compRef.instance.data = componentState.data;
      compRef.instance.testId = this.test.id;

      container["compRef"] = compRef;

      compRef.changeDetectorRef.detectChanges();
    });

    this.layout.registerComponent("questInst", (container, componentState) => {
      const factory = this.componentFactoryResolver.resolveComponentFactory(QuestInstComponent);

      const compRef = viewContainer.createComponent(factory);
      // compRef.instance.setEventHub(this.layout.eventHub);
      container.getElement().append(compRef.location.nativeElement);
      compRef.instance.data = componentState.data;
      compRef.instance.testId = this.test.id;

      container["compRef"] = compRef;

      compRef.changeDetectorRef.detectChanges();
    });

    this.layout.registerComponent("video", (container, componentState) => {
      const factory = this.componentFactoryResolver.resolveComponentFactory(VideoComponent);

      const compRef = viewContainer.createComponent(factory);
      // compRef.instance.setEventHub(this.layout.eventHub);
      container.getElement().append(compRef.location.nativeElement);
      compRef.instance.video = componentState.data;
      container["compRef"] = compRef;

      compRef.changeDetectorRef.detectChanges();
    });

    this.layout.registerComponent("photo", (container, componentState) => {
      const factory = this.componentFactoryResolver.resolveComponentFactory(PhotoComponent);

      const compRef = viewContainer.createComponent(factory);
      // compRef.instance.setEventHub(this.layout.eventHub);
      container.getElement().append(compRef.location.nativeElement);
      compRef.instance.data = componentState.data;
      container["compRef"] = compRef;

      compRef.changeDetectorRef.detectChanges();
    });

    this.layout.registerComponent("timeline", (container, componentState) => {
      const factory = this.componentFactoryResolver.resolveComponentFactory(TimelineComponent);

      const compRef = viewContainer.createComponent(factory);
      // compRef.instance.setEventHub(this.layout.eventHub);
      compRef.instance.inst = componentState.instance;
      compRef.instance.test = this.test;


      container.getElement().append(compRef.location.nativeElement);

      container["compRef"] = compRef;

      compRef.changeDetectorRef.detectChanges();
    });

    this.layout.registerComponent("log", (container, componentState) => {
      const factory = this.componentFactoryResolver.resolveComponentFactory(LogsComponent);

      const compRef = viewContainer.createComponent(factory);
      // compRef.instance.setEventHub(this.layout.eventHub);
      compRef.instance.markers = this.testSvc.markers;
      compRef.instance.log = componentState.data;
      compRef.instance.test = this.test;
      container.getElement().append(compRef.location.nativeElement);

      container["compRef"] = compRef;

      compRef.changeDetectorRef.detectChanges();
    });

    this.layout.registerComponent("text", (container, componentState) => {
      const factory = this.componentFactoryResolver.resolveComponentFactory(TextComponent);

      const compRef = viewContainer.createComponent(factory);
      // compRef.instance.setEventHub(this.layout.eventHub);
      compRef.instance.text = componentState.data;
      container.getElement().append(compRef.location.nativeElement);

      container["compRef"] = compRef;

      compRef.changeDetectorRef.detectChanges();
    });

    this.layout.init();

    this.layout.on("itemDestroyed", item => {
      if (item.container != null) {
        const compRef = item.container["compRef"];
        if (compRef != null) {
          compRef.destroy();
        }
      }
    });
  }
}
